// Version 5.30  19.02.2006 Markus Greither
// Copyright (c) 1991-2006 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efeditsr.h
    \brief Header for the function editor with search/replace functionality
*/

#ifndef __EFEDITSR__
#define __EFEDITSR__

#include "wx/fdrepdlg.h"

// ----------------------------------------------------------------------------
// TEFEditSearch
// ----------------------------------------------------------------------------

//! Class for extension of TEFEdit with find/replace functionality
/*!
    Also updates the programs status bar with current position.
*/
class TEFEditSearch : public TEFEdit
{
  //! Find / replace modes
  enum ReplaceMode
  {
    Replace,
    ReplaceAll,
    Find
  };
  //! Data for find/replace dialog
  wxFindReplaceData SearchData;
  //! Pointer to find/replace dialog
  wxFindReplaceDialog* SearchDialog;

 protected:
  //! Responds to context menu event
  void OnContextMenu(wxMouseEvent &event);
  //! Responds to left button down event
  void OnLButtonDown(wxMouseEvent &event);
  //! Responds to key down event
  void OnKeyDown(wxKeyEvent &event);
  //! Responds to set focus event
  void OnSetFocus(wxFocusEvent &event);
  //! Responds to kill focus event
  void OnKillFocus(wxFocusEvent &event);
  //! Responds to find event
  void OnEditFind(wxCommandEvent& ce);
  //! Responds to find event
  void OnEditFindNext(wxFindDialogEvent& ce);
  //! Responds to replace event
  void OnEditReplace(wxCommandEvent& ce);
  //! Enable / disable find replace events
  void CeEditFindReplace(wxUpdateUIEvent& ce);
  //! Responds to find replace event
  void OnFindReplace(wxFindDialogEvent &ce);
  //! Responds to close of find/replace dialog event
  void OnFindClose(wxFindDialogEvent &ce);
  //! Responds to replace all event
  void OnFindReplaceAll(wxFindDialogEvent &ce);
  //! Show caret pos in status bar
  void ShowCaretPos();

 public:
  //! Ctor
  TEFEditSearch() : TEFEdit()
  {
  }
  //! Ctor
  TEFEditSearch(wxWindow *parent,wxWindowID id,const wxPoint &pos,
                const wxSize &size,
                long style,const wxString &title,
                const NumListFormat &LDV,
                EditMode DialMode = TEFEdit::FunktionStyle);
  //! Dtor
  virtual ~TEFEditSearch();
  //! Set selection and update status bar
  bool SetSelection(unsigned int,unsigned int,bool Scroll = true);
  //! Execute search
  void DoSearch(ReplaceMode Mode);
  //! Invoke find/replace
  void EditFindReplace(ReplaceMode Mode);

 private:
  //! Disable copy Ctor
  TEFEditSearch(const TEFEditSearch&);
  //! Disable copy operator
  TEFEditSearch& operator = (const TEFEditSearch&);

  DECLARE_EVENT_TABLE()
  DECLARE_DYNAMIC_CLASS(TEFEditSearch)
};

#endif

