// Version 5.33  31.03.2007 Markus Greither
// Copyright (c) 1991-2007 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efeqatns.h
    \brief Header for the equations window class
*/

#ifndef __EFEQATNS__
#define __EFEQATNS__

// ----------------------------------------------------------------------------
// TEFEquationsControl
// ----------------------------------------------------------------------------

//! Class for managed TEFEdit control
/*!
    This allows handling of the resize elements around the Editor
    on focus events.
*/
class TEFChildEdit : public TEFEditSearch
{
 public:
  //! Ctor
  TEFChildEdit() : TEFEditSearch()
  {
  }
  //! Ctor
  TEFChildEdit(wxWindow *parent,wxWindowID id,const wxPoint &pos,
                const wxSize &size,
                long style,const wxString &title,
                const NumListFormat &LDV,
                EditMode DialMode = TEFEdit::FunktionStyle)
   : TEFEditSearch(parent,id,pos,size,style,title,LDV,DialMode) {};

  //! Enable parent sizers on set focus
  void OnSetFocus(wxFocusEvent &);

  //! Disable parent sizers on set focus
  void OnKillFocus(wxFocusEvent &);

  //! Shows context menu
  void OnContextMenu(wxMouseEvent &event);
  
  DECLARE_EVENT_TABLE()
  DECLARE_DYNAMIC_CLASS(TEFChildEdit)
};

//! Class for Equations Control
/*!
    This control is a TEFChildControl for moving and resizing
    with an embedded TEFEdit control for equation input.
*/
class TEFEquationsControl : public TEFTabParentControl
{
  //! Cast managed child and return as TEFEdit
  TEFEdit *Editor()
  {
    TEFEdit *tmp = wxDynamicCast(m_child,TEFEdit);
    return tmp;
  }

 public:
  //! Ctor
  TEFEquationsControl() : TEFTabParentControl() {};
  //! Ctor
  TEFEquationsControl(wxWindow *AParent, int AnId,const wxPoint &pos,
                      const wxSize &size,long style,
                      const NumListFormat &LDV,
                      bool ScrollTrack);
  //! Create the window
  void Create(wxWindow *AParent, int AnId,const wxPoint &pos,
              const wxSize &size,long style,
              const NumListFormat &LDV,
              bool ScrollTrack);
  //! Initialize window
  void Init(const NumListFormat &LDV,bool ScrollTrack,const wxSize &size);
  //! Responds to key presses, activates Editor
  void OnKeyDown(wxKeyEvent &event);
  //! Copy editor data into data
  //! Responds to paint event
  void OnPaint(wxPaintEvent &);
  //! Get editor data
  void GetData(MGPStr &data);
  //! Set editor data
  void SetData(const MGPStr &data)
  {
    Editor()->Insert(data.c_str(),true,false,false);
  };
  //! Paint function for printing
  void Paint(wxDC &dc,bool Printing,wxSize &size);
  //! Set editor zoom factor
  void SetZoom(double zoom)
  {
    if (Editor())
      Editor()->SetZoom(zoom);
  }
  //! Returns modified state of editor
  bool GetModified()
  {
    if (Editor())
      return Editor()->IsModified();
    else
      return false;
  }
  //! Clear editor modified state
  void ClearModify()
  {
    if (Editor())
      Editor()->ClearModify();
  }
  //! Returns pointer to embedded editor
  TEFEdit *GetEditor() {return Editor();};
  //! Return current ratio for resize
  float GetRatio()
  {
    wxSize size = GetClientSize();
    return float(size.x)/float(size.y);
  }

  DECLARE_EVENT_TABLE()
  DECLARE_DYNAMIC_CLASS(TEFEquationsControl)
};

#endif

