// Version 5.20  30.06.2003 Markus Greither
// Copyright (c) 1991-2003 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efxrcres.h
    \brief Header for the XRC based resources
*/

#ifndef __EFXRCRES__
#define __EFXRCRES__

// ----------------------------------------------------------------------------
// EFZoomBoxXmlHandler
// ----------------------------------------------------------------------------

//! Class for loading EFZoomBox-objects from XRC resources
class EFZoomBoxXmlHandler : public wxComboBoxXmlHandler
{
public:
    //! Ctor
    EFZoomBoxXmlHandler();
    //! Create resource
    virtual wxObject *DoCreateResource();
    //! Determine if node is our data type
    virtual bool CanHandle(wxXmlNode *node);
};

// ----------------------------------------------------------------------------
// EFToolBarXmlHandler
// ----------------------------------------------------------------------------

//! Class for loading toolbars without attaching them to the frame
class EFToolBarXmlHandler : public wxToolBarXmlHandler
{
public:
    //! Ctor
    EFToolBarXmlHandler()
      : wxToolBarXmlHandler(),m_isInside(false),m_toolbar(0) { };
    //! Create ressource
    virtual wxObject *DoCreateResource();
    //! Determine if node is our data type
    virtual bool CanHandle(wxXmlNode *node);

private:
    //! Flag for current toolbar
    bool m_isInside;
    //! Pointer to current toolbar
    wxToolBar *m_toolbar;
};

// ----------------------------------------------------------------------------
// EFXmlResource
// ----------------------------------------------------------------------------

//! Xrc Resource object, overloaded to be able to load toolbars without setting them to the frame
class EFXmlResource : public wxXmlResource
{
    //! Resource instance
    static EFXmlResource *ef_instance;

 public:
    //! Loads a toolbar without attaching it to the parent
    wxToolBar *LoadEFToolBar(wxWindow *parent, const wxString& name)
    {
        return (wxToolBar*)CreateResFromNode(FindResource(name, wxT("EFToolBar")), parent, NULL);
    }
    //! Gets the global resources object or creates one if none exists.
    static EFXmlResource *Get()
    {
        if ( !ef_instance )
            ef_instance = new EFXmlResource;
#if (wxMAJOR_VERSION == 2) && (wxMINOR_VERSION == 3) && (wxRELEASE_NUMBER < 3)
        wxTheXmlResource = ef_instance;
#else
        Set(ef_instance);
#endif
        return ef_instance;
    }
};

#endif

