// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file cpxpsr.cpp
    \brief Source code for the function parser object and related items.
*/

#ifdef __BORLANDC__

#include "efprec.h"

#pragma hdrstop
#else

#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <set>
#include <functional>
#include <cctype>
#include <cfloat>
#include <complex>
#include <limits>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"

#endif

#if defined(__BORLANDC__) && defined(__WIN32__)
#include <excpt.h>
#endif

// ----------------------------------------------------------------------------
// Variables
// ----------------------------------------------------------------------------

//! Global error flag for matherror
int AkErr = 0;

// ----------------------------------------------------------------------------
// Functions
// ----------------------------------------------------------------------------

#if defined(__DMC__) || defined(_MSC_VER) || defined(__MINGW32__)
#define FPU_FLAGS _EM_DENORMAL | _EM_UNDERFLOW | _EM_INEXACT
#define FPU_MASK _MCW_EM
#define FPU_RESM _MCW_PC
#define FPU_RES _PC_64
#elif defined(__BORLANDC__)
#define FPU_FLAGS EM_DENORMAL | EM_UNDERFLOW | EM_INEXACT | EM_OVERFLOW | EM_INVALID
#define FPU_MASK MCW_EM
#define FPU_RESM MCW_PC
#define FPU_RES PC_64
#endif

//! Reset the floating point unit in case of an overflow
void SIGNAL_CALLING_CONVENTION FPECatcher(int *)
{
  signal(SIGFPE,(fptr)FPECatcher);
#if defined(__BORLANDC__) || defined(__DMC__) || defined (_MSC_VER) || defined(__MINGW32__)
  _fpreset();
#if defined(__DMC__) || defined(_MSC_VER) || defined(__MINGW32__)
  _control87(FPU_MASK,FPU_MASK);
  _control87(FPU_RES,FPU_RESM);
#endif
#endif
}

//! Reset the floating point unit in case of an overflow
void SIGNAL_CALLING_CONVENTION FPEThrowCatcher(int *)
{
  signal(SIGFPE,(fptr)FPEThrowCatcher);
#if defined(__BORLANDC__) || defined(__DMC__) || defined (_MSC_VER) || defined(__MINGW32__)
  _fpreset();
#if defined(__BORLANDC__) || defined(__DMC__) || defined(_MSC_VER)// || defined(__MINGW32__)
  _control87(FPU_FLAGS,FPU_MASK);
  _control87(FPU_RES,FPU_RESM);
#endif
#endif
  // can't throw exceptions in signal handler under GCC, VC handles C exceptions in try/catch
#if defined(__BORLANDC__)
  throw "overflow";
#endif
}

// Set up signal() for FPE exceptions
fptr SetFPESignal()
{
#if defined(__BORLANDC__) || defined(__DMC__) || defined (_MSC_VER) || defined(__MINGW32__)
  _fpreset();
#endif
#if defined(__BORLANDC__) || defined(__DMC__) || defined(_MSC_VER)// || defined(__MINGW32__)
  _control87(FPU_FLAGS,FPU_MASK);
  _control87(FPU_RES,FPU_RESM);
#endif
  return signal(SIGFPE,(fptr)FPEThrowCatcher);
}

// Clear signal() for FPE exceptions
fptr ClearFPESignal(fptr oldhndl)
{
#if defined(__BORLANDC__) || defined(__DMC__) || defined (_MSC_VER) || defined(__MINGW32__)
  _fpreset();
#endif
#if defined(__BORLANDC__) || defined(__DMC__) || defined(_MSC_VER)// || defined(__MINGW32__)
  _control87(FPU_FLAGS,FPU_MASK);
  _control87(FPU_RES,FPU_RESM);
#endif
  return signal(SIGFPE,oldhndl);
}

//! Math error routine, called from math library
/*! Stores error flag in global AkErr
*/
#if defined(__BORLANDC__) || defined(_MSC_VER) || defined(__MINGW32__)
int _matherr(struct _exception *e)
#elif defined (__DMC__)
int __CLIB matherr(struct _exception *e)
#else
int matherr(struct __exception *e)
#endif
{
  if (e->type == UNDERFLOW)
  {
	e->retval = 0;
    return 1;
  }
  AkErr = e->type;
#if defined(__BORLANDC__) || defined(__DMC__) || defined (_MSC_VER)
  _fpreset();
#endif
  return 1;
}


