// Version 6.06  19.01.2010 Markus Greither
// Copyright (c) 1991-2010 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efdlgs.cpp
    \brief Source code for the dialogs not related to a specific window
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h>

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>

#include <wx/docview.h>
#include <wx/clipbrd.h>
#include <wx/notebook.h>
#include <wx/spinctrl.h>
#include <wx/spinctrl.h>

#include <complex>
#include <limits>
#include <cstdio>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "efsolver.h"
#include "efedit.h"
#include "efeditsr.h"
#include "efdocum.h"
#include "ef.h"
#include "eftabcon.h"
#include "efgraphc.h"
#include "efresid.h"
#include "effnkclc.h"
#include "efview.h"
#include "efdlgs.h"

#endif

#include <wx/fontdlg.h>
#include <wx/colordlg.h>
#include <wx/html/htmlwin.h>
#include "wx/hyperlink.h"

// ----------------------------------------------------------------------------
// TEFComboBox
// ----------------------------------------------------------------------------

void TEFComboBox::GetComboData(EFComboData &CD)
{
  MGPStr tmp;
  ZahlFormatObj ZF(wxGetApp().GetLDV());
  bool test = false;

  wxString tmp1 = GetValue(); // Hole Inhalt der Eingabezeile
  tmp.SetStr(tmp1.c_str()); // In tmp kopieren
  ZF.WinToStd(tmp); // in Standart-Format umwandeln
  tmp1 = tmp.c_str(); // Zurueck in Zeile
  int f = GetCount(); // Anzahl der Zeilen in ListBox
  for (int i = 0;i < f;i++)
  { // Zeilen einlesen
    wxString tmp2 = GetString(i);
    tmp.SetStr(tmp2.c_str());
    ZF.WinToStd(tmp);
    // pruefen ob String aus Eingabezeile auch in ListBox
    test |= (_tcscmp(tmp1.c_str(),tmp.c_str()) == 0);
    CD.SetString(i,tmp.c_str());
  }
  if (!test && tmp1.Length())
  { // wenn Zeile noch nicht vorhanden, hinten anhaengen
    if (f < EFComboData::MaxComboItems)
      CD.SetString(f,tmp1.c_str());
    else
    {
      for (int i = 0;i < EFComboData::MaxComboItems-1;i++)
        CD.SetString(i,CD.GetString(i+1));
      CD.SetString(EFComboData::MaxComboItems-1,tmp1.c_str());
    }
  }
}

void TEFComboBox::SetComboData(EFComboData &CD)
{
  MGPStr tmp;
  ZahlFormatObj ZF(wxGetApp().GetLDV());
  Clear(); // Combo Box leeren
  for (int i = 0;i < CD.GetCount();i++)
  {
    tmp.SetStr(CD.GetString(i));
    ZF.StdToWin(tmp);
    if (FindString(wxString(tmp.c_str())) < 0)
      Append(wxString(tmp.c_str()));
  }
}

// ----------------------------------------------------------------------------
// TEFEditDlg
// ----------------------------------------------------------------------------

TEFEditDlg::TEFEditDlg() : TEFHilfeDlg()
{
  ButtonsEnabled = false;
  OK = 0;
}

TEFEditDlg::~TEFEditDlg()
{
  delete Editor;
}

void TEFEditDlg::Create(wxWindow *parent,wxWindowID Id,const wxString &title)
{
//  TEFHilfeDlg::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgEdit"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Editor = new TEFEdit(this,XRCID("ID_SOLVEDIT"),wxPoint(0,0),wxSize(-1,300),
                       wxSUNKEN_BORDER | wxCLIP_CHILDREN |
                       wxNO_FULL_REPAINT_ON_RESIZE |
                       wxWANTS_CHARS | wxVSCROLL | wxHSCROLL,wxT("EF-Edit"),
                       wxGetApp().GetLDV(),TEFEdit::DialogStyle);
  Editor->SetViewOption(wxGetApp().EditFlags & MyApp::EditScrollTrack,
                        TEFEdit::ViewScrollTrack,false);
  Editor->SetLabelFontStyle(wxGetApp().LFontStyle);
  Editor->SetKommentFontStyle(wxGetApp().KFontStyle);
  Editor->SetLabelFontColour(wxGetApp().LFontColour);
  Editor->SetKommentFontColour(wxGetApp().KFontColour);
  Editor->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
  Editor->ClearEditor();
  wxXmlResource::Get()->AttachUnknownControl(wxT("ID_EDITWINDOW"),Editor);
  Editor->Layout();
  OK = FindB(wxID_OK);
  Cancel = FindB(wxID_CANCEL);
  Kopie = FindB(wxID_NO);
  Einfg = FindB(wxID_YES);
  Druck = FindB(XRCID("IDIGNORE"));
  Centre();
  EnableButtons(true);
}

void TEFEditDlg::EnableButtons(bool A)
{
  if (A != ButtonsEnabled)
  {
    ButtonsEnabled = A;
    if (ButtonsEnabled == false)
    {
      OK->Enable(ButtonsEnabled);
      Einfg->Enable(ButtonsEnabled);
      Druck->Enable(ButtonsEnabled);
    }
    OK->Enable(ButtonsEnabled);
  }
}

void TEFEditDlg::OnIgnore(wxCommandEvent &)
{ // Drucken
  Editor->SetFocus();
  wxView *view = wxGetApp().m_docManager->GetCurrentView();
  wxCommandEvent cevent;
  if (view)
    ((TEFView *)view)->OnPrintFocusWindow(cevent);
}

void TEFEditDlg::OnNo(wxCommandEvent &)
{ // Kopieren
  unsigned int Anf,End;
  Editor->GetSelection(Anf,End);
  if (End == Anf)
    Editor->SetSelection(0,Editor->GetTextLen());
  Editor->Copy();
}

void TEFEditDlg::OnYes(wxCommandEvent &)
{ // Einfuegen
  Editor->Paste();
}

void TEFEditDlg::OnIdle(wxIdleEvent &event)
{
#ifdef __WXGTK__
  if (wxTheClipboard->IsOpened())
    return;
#endif
  if (IsShown()) // necessary on wxGTK, otherwise crash after Destroy() (?!)
  {  
    if (wxTheClipboard->Open())
    {
      if (wxTheClipboard->IsSupported(wxDF_TEXT))
        Einfg->Enable(ButtonsEnabled);
      else
        Einfg->Enable(false);
      wxTheClipboard->Close();
    }
    Kopie->Enable(true);
    OK->Enable(ButtonsEnabled);
    Druck->Enable(ButtonsEnabled);
  }
  event.Skip();
}

void TEFEditDlg::OnSize(wxSizeEvent &event)
{
  event.Skip();
  if (!OK)
    return;
  OK->Refresh();
  Kopie->Refresh();
  Einfg->Refresh();
  Druck->Refresh();
  Cancel->Refresh();
  FindWindow(wxID_HELP)->Refresh();
}

BEGIN_EVENT_TABLE(TEFEditDlg,TEFHilfeDlg)
  EVT_MENU(wxID_COPY,TEFEditDlg::OnEditCopy) // fuer PopupMenu
  EVT_MENU(wxID_PASTE,TEFEditDlg::OnEditPaste)
  EVT_MENU(wxID_CUT,TEFEditDlg::OnEditCut)
  EVT_MENU(XRCID("CMD_MARKALL"),TEFEditDlg::OnMarkAll)
  EVT_IDLE(TEFEditDlg::OnIdle)
  EVT_SIZE(TEFEditDlg::OnSize)
  EVT_BUTTON(wxID_NO,TEFEditDlg::OnNo)
  EVT_BUTTON(wxID_YES,TEFEditDlg::OnYes)
  EVT_BUTTON(XRCID("IDIGNORE"), TEFEditDlg::OnIgnore)
  EVT_BUTTON(wxID_CANCEL, TEFEditDlg::OnCancel)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// EFHilfeDlg
// ----------------------------------------------------------------------------

void TEFHilfeDlg::OnHilfe(wxCommandEvent &)
{
  wxGetApp().HelpCtrl.DisplaySection(GetId());
}

void TEFHilfeDlg::OnIdle(wxIdleEvent &event)
{
  wxWindow *focus = FindFocus();
  if (focus && IsShown()) // use IsShown() to prevent evtl. crashes under wxGTK
  {
    wxButton *btn = wxDynamicCast(focus,wxButton);
    wxButton *defbtn = wxDynamicCast(defbutton,wxButton);
    if (!btn && defbutton && (GetDefaultItem() != defbutton) && defbtn)
      defbtn->SetDefault();
  }
  event.Skip();
}

void TEFHilfeDlg::GetColour(wxWindow *Wd,unsigned long &Farbe,
                               const wxChar *Titel)
{
  wxColourDialog *TC;
  wxColourData TD;
  int RetStatus;

  TD.SetChooseFull(false);
  wxColour orgcol(Farbe & 0xFF,(Farbe >> 8) & 0xFF,(Farbe >> 16) & 0xFF);
  TD.SetColour(orgcol);
  TC = new wxColourDialog(Wd,&TD);
  TC->SetTitle(Titel);
  if (TC)
  {
    RetStatus = TC->ShowModal();
    TD = TC->GetColourData();
    delete TC;
    if (RetStatus == wxID_OK)
      Farbe = (unsigned long)(TD.GetColour().Red() |
                             (TD.GetColour().Green() << 8) |
                             (TD.GetColour().Blue() << 16));
  }
}

void TEFHilfeDlg::GetFont(wxWindow *Wd,FontType &Font,const wxChar *Titel)
{
  wxFontData TD;
  wxFont font(Font.Size,wxDEFAULT,
              Font.Style & FontType::Italic ? wxITALIC : wxNORMAL,
              Font.Style & FontType::Bold ? wxBOLD : wxNORMAL,
              Font.Style & FontType::Underline,Font.Font.c_str());
  TD.SetInitialFont(font);
  TD.SetColour(wxColour(Font.Color & 0xFF,
                        (Font.Color >> 8) & 0xFF,
                        (Font.Color >> 16) & 0xFF));
  wxFontDialog dialog(Wd,TD);
  dialog.SetTitle(Titel);
  if (dialog.ShowModal() == wxID_OK)
  {
    TD = dialog.GetFontData();
    wxFont newfont = TD.GetChosenFont();
    Font.Size = newfont.GetPointSize();
    wxString name = newfont.GetFaceName();
    Font.Font.SetStr(name.c_str());
    Font.Style = (newfont.GetWeight() == wxBOLD)*FontType::Bold+
                 (newfont.GetStyle() == wxITALIC)*FontType::Italic+
                 (newfont.GetUnderlined())*FontType::Underline;
    Font.Color = (unsigned long)(TD.GetColour().Red() |
                                 (TD.GetColour().Green() << 8) |
                                 (TD.GetColour().Blue() << 16));
  }
}

BEGIN_EVENT_TABLE(TEFHilfeDlg,wxDialog)
  EVT_BUTTON(wxID_HELP,TEFHilfeDlg::OnHilfe)
  EVT_IDLE(TEFHilfeDlg::OnIdle)
END_EVENT_TABLE()


// ----------------------------------------------------------------------------
// TEFUmformenDlg
// ----------------------------------------------------------------------------

TEFUmformenDlg::TEFUmformenDlg(UmformenData &D)
  : TEFHilfeDlg(), Data(D)
{
}

TEFUmformenDlg::~TEFUmformenDlg()
{
  delete Erg;
  delete Par;
  delete Zus;
}

void TEFUmformenDlg::Create(wxWindow *parent,wxWindowID Id,
                            const wxString &title)
{
//  wxDialog::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgUmformen"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  Erg = (TEFComboBox *)FindWindow(XRCID("ID_ERGEBNIS"));
  Par = (TEFComboBox *)FindWindow(XRCID("ID_PARAMETER"));
  Zus = (TEFComboBox *)FindWindow(XRCID("ID_ERGEBNIS1"));
  Tem = (TEFComboBox *)FindWindow(XRCID("ID_TEMPLATE"));
  Erg->SetComboData(*Data.ErgH);
  Par->SetComboData(*Data.ParH);
  Zus->SetComboData(*Data.ZusH);
  wxComboBox *cb = FindCoB(XRCID("ID_TEMPLATE"));
  cb->Clear();
  Data.Vorlage->Begin();
  while (Data.Vorlage->NextEint())
    cb->Append((*Data.Vorlage)->Name.c_str());
  AkDataIndex = 0;
  SetData(AkDataIndex);
  Fit();
  FindWindow(XRCID("ID_TEMPLATE"))->SetFocus();
}

void TEFUmformenDlg::OnTemplate(wxCommandEvent &)
{
  wxComboBox *cb = FindCoB(XRCID("ID_TEMPLATE"));
  AkDataIndex = cb->GetSelection()+1;
  SetData(AkDataIndex);
}

void TEFUmformenDlg::OnDelTemplate(wxCommandEvent &)
{
  if (AkDataIndex > 0)
  {
    wxComboBox *cb = FindCoB(XRCID("ID_TEMPLATE"));
    cb->Delete(AkDataIndex-1);
    Data.Vorlage->SetAkEint(AkDataIndex);
    Data.Vorlage->LoeschAkEint();
    AkDataIndex = 0;
    SetData(AkDataIndex);
  }
}

void TEFUmformenDlg::SetData(int i)
{
  Data.Vorlage->SetAkEint(i);
  Data.AkSett = Data.Vorlage->GetAkEint();
  Erg->SetValue(Data.AkSett.ErgStr.c_str());
  Par->SetValue(Data.AkSett.BeibehaltenStr.c_str());
  Zus->SetValue(Data.AkSett.ZusammenStr.c_str());
  Tem->SetValue(Data.AkSett.Name.c_str());
  wxString tmp = wxT("ID_SW2");
  tmp += wxChar(Data.AkZeileOpt+48);
  FindRB(wxXmlResource::Get()->GetXRCID(tmp))->SetValue(true);
  FindCBX(XRCID("ID_SW10"))->SetValue(Data.AkSett.Mode);
  FindCBX(XRCID("ID_GLANH"))->SetValue(Data.AkSett.Anh & EFDaten::UAnh);
  FindCBX(XRCID("ID_GLDEBUG"))->SetValue(Data.AkSett.Anh & EFDaten::UDebug);
  FindCBX(XRCID("ID_SW25"))->SetValue(Data.AkSett.Anh & EFDaten::UKlKuerzen);
  FindCBX(XRCID("ID_COMMENT"))->SetValue(Data.AkSett.Anh & EFDaten::UComment);
  FindCBX(XRCID("ID_RULES"))->SetValue(Data.AkSett.Anh & EFDaten::URules);
  Data.AkSett.ParamsList.Begin();
  FindTC(XRCID("ID_DEFINITIONS"))->Clear();
  if (Data.AkSett.ParamsList.GetMaxEintrag() >= 0)
    do
    {
      const _TCHAR *tmp = Data.AkSett.ParamsList.GetAkEint().c_str();
      FindTC(XRCID("ID_DEFINITIONS"))->AppendText(tmp);
      FindTC(XRCID("ID_DEFINITIONS"))->AppendText(wxT("\n"));
    }
    while (Data.AkSett.ParamsList.NextEint());
  FindWindow(XRCID("ID_DELTEMPLATE"))->Enable(AkDataIndex > 0);
}

void TEFUmformenDlg::OnCancel(wxCommandEvent &)
{
  Erg->GetComboData(*Data.ErgH);
  Par->GetComboData(*Data.ParH);
  Zus->GetComboData(*Data.ZusH);
  EndModal(wxID_CANCEL);
}

void TEFUmformenDlg::OnOk(wxCommandEvent &)
{
  int i;
  wxString tmp;

  tmp = Erg->GetValue();
  Data.AkSett.ErgStr.SetStr(tmp.c_str());
  tmp = Par->GetValue();
  Data.AkSett.BeibehaltenStr.SetStr(tmp.c_str());
  tmp = Zus->GetValue();
  Data.AkSett.ZusammenStr.SetStr(tmp.c_str());
  tmp = Tem->GetValue();
  Data.AkSett.Name.SetStr(tmp.c_str());
  Erg->GetComboData(*Data.ErgH);
  Par->GetComboData(*Data.ParH);
  Zus->GetComboData(*Data.ZusH);
  tmp = wxT("ID_SW2");
  for (i = 0;i <= 1;i++)
  {
    tmp += wxChar(i+48);
    if (FindRB(wxXmlResource::Get()->GetXRCID(tmp))->GetValue())
      Data.AkZeileOpt = (unsigned short)(i);
    tmp.RemoveLast();
  }
  Data.AkSett.Mode = FindCBX(XRCID("ID_SW10"))->GetValue();
  Data.AkSett.Anh = FindCBX(XRCID("ID_SW25"))->GetValue()*EFDaten::UKlKuerzen +
                    FindCBX(XRCID("ID_GLDEBUG"))->GetValue()*EFDaten::UDebug +
                    FindCBX(XRCID("ID_GLANH"))->GetValue()*EFDaten::UAnh +
                    FindCBX(XRCID("ID_COMMENT"))->GetValue()*EFDaten::UComment +
                    FindCBX(XRCID("ID_RULES"))->GetValue()*EFDaten::URules;
  i = 0;
  Data.AkSett.ParamsList.ResetKette();
  while (i <= FindTC(XRCID("ID_DEFINITIONS"))->GetNumberOfLines())
  {
    wxString tmp = FindTC(XRCID("ID_DEFINITIONS"))->GetLineText(i);
    if (!tmp.IsEmpty())
    {
      Data.AkSett.ParamsList.NeuAkEint();
      Data.AkSett.ParamsList->SetStr(tmp.c_str());
    }
    i++;
  }
  EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE( TEFUmformenDlg, TEFHilfeDlg )
    EVT_BUTTON(wxID_OK, TEFUmformenDlg::OnOk)
    EVT_BUTTON(wxID_CANCEL, TEFUmformenDlg::OnCancel)
    EVT_BUTTON(XRCID("ID_DELTEMPLATE"), TEFUmformenDlg::OnDelTemplate)
    EVT_COMBOBOX(XRCID("ID_TEMPLATE"), TEFUmformenDlg::OnTemplate)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFNaeherungDlg
// ----------------------------------------------------------------------------

TEFNaeherungDlg::TEFNaeherungDlg(NaeherungDataType &D)
  : TEFHilfeDlg(), Data(D)
{
}

TEFNaeherungDlg::~TEFNaeherungDlg()
{
  delete Param;
}

void TEFNaeherungDlg::Create(wxWindow *parent,wxWindowID Id,
                             const wxString &title)
{
//  wxDialog::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgNaeherung"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  Param = FindLB(XRCID("ID_PARAMETER"));
  wxString tmp = wxT("ID_SW1");
  tmp += wxChar(Data.Mode+48);
  if (Data.Mode != 1)
    FindRB(wxXmlResource::Get()->GetXRCID(tmp))->SetValue(true);
  ((wxNotebook *)FindWindow(XRCID("ID_PARAMBK")))->SetSelection(Data.Mode == 1 ? 1 : 0);
  FindCBX(XRCID("ID_CURGRAPH"))->SetValue(Data.m_CurGraph);
  FindTC(XRCID("ID_ERGEBNIS"))->SetValue(wxString(Data.ErgStr.c_str()));
  int AltPos,i = 0;
  do
  {
    while (Data.ParamStr[i] && (Data.ParamStr[i] == ' '))
      i++;
    AltPos = i;
    while (Data.ParamStr[i] && (Data.ParamStr[i] != ' '))
      i++;
    if (i > AltPos)
    {
      MGPStr tmp;
      tmp.AddNChar(&Data.ParamStr[AltPos],i-AltPos);
      Param->Append(tmp.c_str());
    }
  }
  while (i > AltPos);
  FindTC(XRCID("ID_X1"))->SetValue(wxString(Data.XStr[0].c_str()));
  FindTC(XRCID("ID_X2"))->SetValue(wxString(Data.XStr[1].c_str()));
  FindTC(XRCID("ID_X3"))->SetValue(wxString(Data.XStr[2].c_str()));
  FindTC(XRCID("ID_Y1"))->SetValue(wxString(Data.YStr[0].c_str()));
  FindTC(XRCID("ID_Y2"))->SetValue(wxString(Data.YStr[1].c_str()));
  FindTC(XRCID("ID_Y3"))->SetValue(wxString(Data.YStr[2].c_str()));
  EnChanged();
  BnClicked();
  Refresh();
  FindSC(XRCID("ID_NUMPTS"))->SetRange(2,!Data.Mode ? 1000 : 20);
  FindSC(XRCID("ID_NUMPTS"))->SetValue(Data.m_NumPts);
  FindTC(XRCID("ID_ERGEBNIS"))->SetFocus();
}

void TEFNaeherungDlg::OnParSel(wxCommandEvent &)
{
  FindTC(XRCID("ID_NEWLINE"))->SetValue(Param->GetStringSelection());
}

void TEFNaeherungDlg::OnYes(wxCommandEvent &)
{
  wxString Zeile = FindTC(XRCID("ID_NEWLINE"))->GetValue();
  if ((!Zeile.IsNull()) && (Param->FindString(Zeile) < 0))
  {
    Param->Append(Zeile);
    FindTC(XRCID("ID_NEWLINE"))->SetValue(wxT(""));
  }
  else
    wxBell();
}

void TEFNaeherungDlg::OnNo(wxCommandEvent &)
{
  int ind = Param->GetSelection();
  if (ind >= 0)
    Param->Delete(ind);
  else
    wxBell();
}

void TEFNaeherungDlg::OnIgnore(wxCommandEvent &)
{
  Param->Clear();
}

void TEFNaeherungDlg::OnCancel(wxCommandEvent &)
{
  EndModal(wxID_CANCEL);
}

void TEFNaeherungDlg::OnOk(wxCommandEvent &)
{
  unsigned int i;
  wxString Zeile = FindTC(XRCID("ID_ERGEBNIS"))->GetValue();
  Data.ErgStr.SetStr(Zeile.c_str());
  Data.ParamStr.SetStr(wxT(""));
  for (i = 0;i < Param->GetCount();i++)
  {
    Zeile = Param->GetString(i);
    Data.ParamStr.AddStr(Zeile.c_str());
    if (i < Param->GetCount()-1)
      Data.ParamStr.AddStr(wxT(" "));
  }
  Zeile = FindTC(XRCID("ID_NEWLINE"))->GetValue();
  if (!Zeile.IsEmpty())
  { // evtl. vorhandenen Eintrag im Eingabefeld "mitnehmen"
    Data.ParamStr.AddStr(wxT(" "));
    Data.ParamStr.AddStr(Zeile.c_str());
  }
  wxString tmp;
  for (i = 0;i < 3;i++)
  {
    tmp = wxT("ID_X");
    tmp += wxChar(i+1+48);
    Zeile = FindTC(wxXmlResource::Get()->GetXRCID(tmp))->GetValue();
    Zeile.Truncate(80);
    Data.XStr[i].SetStr(Zeile.c_str());
    tmp = wxT("ID_Y");
    tmp += wxChar(i+1+48);
    Zeile = FindTC(wxXmlResource::Get()->GetXRCID(tmp))->GetValue();
    Zeile.Truncate(80);
    Data.YStr[i].SetStr(Zeile.c_str());
  }
  tmp = wxT("ID_SW1");
  for (i = 0;i <= 5;i++)
  {
    if (i == 1)
      i++;
    tmp += wxChar(i+48);
    if (FindRB(wxXmlResource::Get()->GetXRCID(tmp))->GetValue())
      Data.Mode = (unsigned short)(i);
    tmp.RemoveLast();
  }
  if (((wxNotebook *)FindWindow(XRCID("ID_PARAMBK")))->GetSelection() == 1)
    Data.Mode = 1;
  Data.m_CurGraph = FindCBX(XRCID("ID_CURGRAPH"))->GetValue();
  Data.m_NumPts = FindSC(XRCID("ID_NUMPTS"))->GetValue();
  EndModal(wxID_OK);
}

void TEFNaeherungDlg::OnEnChanged(wxCommandEvent &)
{
  EnChanged();
}

void TEFNaeherungDlg::EnChanged()
{
  MGPStr Zeile;
  ZahlFormatObj ZF(wxGetApp().GetLDV());
  wxString Str1 = FindTC(XRCID("ID_X1"))->GetValue();
  wxString Str2 = FindTC(XRCID("ID_X2"))->GetValue();
  double X1 = ZF.ZahlWert(Str1.c_str());
  double X2 = ZF.ZahlWert(Str2.c_str());
  double X3 = X2+(X2-X1);
  ZF.Ausgabe(Zeile,X3,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_X3"))->SetValue(Zeile.c_str());
}

void TEFNaeherungDlg::OnBnClicked(wxCommandEvent &)
{
  BnClicked();
}

void TEFNaeherungDlg::BnClicked()
{
  Data.m_CurGraph = (unsigned short)FindCBX(XRCID("ID_CURGRAPH"))->GetValue();
  FindWindow(XRCID("ID_X3"))->Enable(false);
  FindWindow(XRCID("ID_PARAMETER"))->Enable(!Data.m_CurGraph);
  FindWindow(XRCID("ID_NEWLINE"))->Enable(!Data.m_CurGraph);
  FindWindow(wxID_YES)->Enable(!Data.m_CurGraph);
  FindWindow(wxID_NO)->Enable(!Data.m_CurGraph);
  FindWindow(XRCID("IDIGNORE"))->Enable(!Data.m_CurGraph);
  FindWindow(XRCID("ID_NUMPTSTEXT"))->Enable(Data.m_CurGraph);
  FindWindow(XRCID("ID_NUMPTS"))->Enable(Data.m_CurGraph);
}

void TEFNaeherungDlg::OnFunctionType(wxCommandEvent &)
{
  int curval = 0;
  wxString tmp = wxT("ID_SW1");
  for (int i = 0;i <= 5;i++)
  {
    if (i == 1)
      i++;
    tmp += wxChar(i+48);
    if (FindRB(wxXmlResource::Get()->GetXRCID(tmp))->GetValue())
      curval = i;
    tmp.RemoveLast();
  }
  FindSC(XRCID("ID_NUMPTS"))->SetRange(2,!curval ? 1000 : 20);
  if (FindSC(XRCID("ID_NUMPTS"))->GetValue() > FindSC(XRCID("ID_NUMPTS"))->GetMax())
  {
    FindSC(XRCID("ID_NUMPTS"))->SetValue(FindSC(XRCID("ID_NUMPTS"))->GetMax());
  }
}

BEGIN_EVENT_TABLE( TEFNaeherungDlg, TEFHilfeDlg )
  EVT_TEXT_ENTER(XRCID("ID_NEWLINE"),TEFNaeherungDlg::OnYes)
  EVT_LISTBOX(XRCID("ID_PARAMETER"), TEFNaeherungDlg::OnParSel)
  EVT_CHECKBOX(XRCID("ID_CURGRAPH"),TEFNaeherungDlg::OnBnClicked)
  EVT_TEXT(XRCID("ID_X1"), TEFNaeherungDlg::OnEnChanged)
  EVT_TEXT(XRCID("ID_X2"), TEFNaeherungDlg::OnEnChanged)
  EVT_BUTTON(wxID_OK, TEFNaeherungDlg::OnOk)
  EVT_BUTTON(wxID_CANCEL, TEFNaeherungDlg::OnCancel)
  EVT_BUTTON(wxID_YES, TEFNaeherungDlg::OnYes)
  EVT_BUTTON(wxID_NO, TEFNaeherungDlg::OnNo)
  EVT_BUTTON(XRCID("IDIGNORE"), TEFNaeherungDlg::OnIgnore)
  EVT_RADIOBUTTON(XRCID("ID_SW10"),TEFNaeherungDlg::OnFunctionType)
  EVT_RADIOBUTTON(XRCID("ID_SW12"),TEFNaeherungDlg::OnFunctionType)
  EVT_RADIOBUTTON(XRCID("ID_SW13"),TEFNaeherungDlg::OnFunctionType)
  EVT_RADIOBUTTON(XRCID("ID_SW14"),TEFNaeherungDlg::OnFunctionType)
  EVT_RADIOBUTTON(XRCID("ID_SW15"),TEFNaeherungDlg::OnFunctionType)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFBerechnenDlg
// ----------------------------------------------------------------------------

TEFBerechnenDlg::TEFBerechnenDlg(wxWindow *parent,wxWindowID Id,
                                 const wxString &title,ParserObj &psr)
  : wxProgressDialog(title,wxT(""),TEFFunktionCalc::GetGaugeRange(),
                     parent,wxPD_CAN_ABORT | wxPD_APP_MODAL),
    Parser(psr)
{
  Data = 0;
  SetId(Id);
  Centre();
  FCD = 0;
  FunktionCalc = 0;
}

void TEFBerechnenDlg::Init(BerechnenData &D)
{
  Data = &D;
  delete FCD;
  FCD = new TEFFunktionCalcData(Data->Fehler);
  FCD->FuName.SetStr(D.FuName);
  FCD->AkFeld = D.AkFeld;
  FCD->DiagData = D.EFGraph->GetDiagData();
  FCD->BerFus = D.BerFus;
  delete FunktionCalc;
  FunktionCalc = new TEFFunktionCalc(&Parser,*FCD);
  wxString AkStr;
  if (FunktionCalc->GetCalcMode() == MaTyp::ModeFourier)
    AkStr = _("Calculate coefficients");
  else
  {
    if ((FunktionCalc->GetCalcMode() == MaTyp::ModePar) ||
        (FunktionCalc->GetCalcMode() == MaTyp::ModeParList))
    {
      AkStr.Printf(_("Value calculation - parameter %i"),
              FunktionCalc->GetAkPar());
    }
    else
    {
       if (FunktionCalc->GetCalcMode() == MaTyp::ModeSum)
         AkStr = _("Value calculation with summing");
       else
         AkStr = _("Value calculation");
    }
  }
  Update(0,AkStr);
}

TEFBerechnenDlg::~TEFBerechnenDlg()
{
  delete FunktionCalc;
  delete FCD;
}

void TEFBerechnenDlg::GraphSpeichern(const _TCHAR *HiString)
{
  int RetStatus;

  MGPStr str;
  if (HiString)
    str = HiString;
  else
    FunktionCalc->GetFunktionName(str);
  if (FunktionCalc->GetPunkte() >= 2)
  {

    FunktionCalc->SetDefaultGraphOptions(str.c_str(),wxGetApp().GraphSymb,
                                         wxGetApp().GraphOpt,
                                         wxGetApp().GraphDarst);
    if (Data->Update)
    {
      if ((!Data->EFGraph->GetDiagData()->YDaten.YDatenVorhanden()) ||
          (!Data->EFGraph->FuStringVorh(str.c_str())))
      {
        wxString tmp;
        tmp.Printf(_("Graph with name '%s' not found, create new graph ?"),
                   str.c_str());
        RetStatus = wxMessageBox(tmp,wxT("Easy Funktion"),
                                 wxYES_NO+wxICON_EXCLAMATION,this);
        if (RetStatus != wxYES)
          return;
      }
      else
      {
        Data->EFGraph->GetDiagData()->YDaten.ResetYDaten();
        if (Data->EFGraph->GetDiagData()->YDaten.GetMaxPos() > 0)
          while ((_tcscmp(Data->EFGraph->GetDiagData()->YDaten.AkGraph().FuString,
                         str.c_str()) != 0) &&
                 (Data->EFGraph->GetDiagData()->YDaten.LoadYFeld()));
        if (_tcscmp(Data->EFGraph->GetDiagData()->YDaten.AkGraph().FuString,
                   str.c_str()) == 0)
          FunktionCalc->UpdateGraph();
        return;
      }
    }
    RetStatus = Data->EFGraph->CheckGraphName(str,this);
    if (RetStatus == wxID_OK)
      FunktionCalc->AppendNewGraph(str.c_str());
  }
}

int TEFBerechnenDlg::ShowModal()
{
  bool Tst;

  if (!Data)
    return wxID_CANCEL;
  double AkHi = FunktionCalc->GetAkHi();
  bool active = true;
  if (!FunktionCalc->GetFinished())
  do
  {
    int OldPar = FunktionCalc->GetAkPar();
    bool OldMode = FunktionCalc->GetFourMode();
    int oldval = FunktionCalc->GetGaugeValue();
    Tst = FunktionCalc->NextStep();
    if (OldPar != FunktionCalc->GetAkPar())
    {
      MGPStr HiString;
      AkHi = FunktionCalc->GetAkHi();
      FunktionCalc->GetFunktionName(HiString,&AkHi);
      GraphSpeichern(HiString.c_str());
      wxString AkStr;
      AkStr.Printf(_("Value calculation - parameter %i"),
              FunktionCalc->GetAkPar());
      active = Update(FunktionCalc->GetGaugeValue(),AkStr);
    }
    else
    if (OldMode != FunktionCalc->GetFourMode())
      active = Update(FunktionCalc->GetGaugeValue(),_("Value calculation"));
    else
    if (oldval != FunktionCalc->GetGaugeValue())
      active = Update(FunktionCalc->GetGaugeValue());
    if (!active)
    {
      GraphSpeichern();
      Data = 0;
      return wxID_CANCEL;
    }
    if (FunktionCalc->GetFinished())
      return Berechnet();
  }
  while (Tst);
  return wxID_OK;
}

int TEFBerechnenDlg::Berechnet()
{
  GraphSpeichern();
  int ret;
  if (Data->Fehler.Art != ParserObj::NoError)
    ret = wxID_CANCEL;
  else
    ret = wxID_OK;
  Data = 0;
  return ret;
}

// ----------------------------------------------------------------------------
// TEFOptionenDlg
// ----------------------------------------------------------------------------

TEFOptionenDlg::TEFOptionenDlg() : TEFHilfeDlg()
{
}

void TEFOptionenDlg::Create(wxWindow *parent,wxWindowID Id,
                            const wxString &title)
{
//  wxDialog::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgOptionen"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  InitControls();
}

void TEFOptionenDlg::InitControls()
{
  FindCBX(XRCID("ID_SW10"))->SetValue(wxGetApp().ZahlForm);
  FindCBX(XRCID("ID_SW21"))->SetValue(wxGetApp().WinZahlFormat);
  FindCBX(XRCID("ID_SW20"))->SetValue((wxGetApp().Options &
                                       MyApp::keyLowerCase) > 0);
  FindCBX(XRCID("ID_WARN_SOLVE"))->SetValue(wxGetApp().Options &
                                            MyApp::keyWarnSolve);
  FindCBX(XRCID("ID_WARN_DELETE"))->SetValue(wxGetApp().Options &
                                             MyApp::keyWarnDelete);
  FindCBX(XRCID("ID_EMFFORMAT"))->SetValue((wxGetApp().Options &
                                            MyApp::keySaveEMFAsBitmap) > 0);
  FindCBX(XRCID("ID_REALRESULTS"))->SetValue((wxGetApp().Options &
                                              MyApp::keyOnlyRealResults) > 0);
  FindCBX(XRCID("ID_FUNCSELVISIBLE"))->SetValue((wxGetApp().Options &
                                                MyApp::keyFuncSelVisible) > 0);
  FindTC(XRCID("ID_ERGEBNIS"))->SetValue(wxGetApp().GetStdFilePath());
  bool Set;
  Set = (wxGetApp().LFontStyle & TEFTextCalc::Bold) ==
         TEFTextCalc::Bold;
  FindCBX(XRCID("ID_LABELBOLD"))->SetValue(Set);
  Set = (wxGetApp().LFontStyle & TEFTextCalc::Italic) ==
         TEFTextCalc::Italic;
  FindCBX(XRCID("ID_LABELITALIC"))->SetValue(Set);
  Set = (wxGetApp().LFontStyle & TEFTextCalc::Underline) ==
         TEFTextCalc::Underline;
  FindCBX(XRCID("ID_LABELUNDERLINE"))->SetValue(Set);
  Set = (wxGetApp().KFontStyle & TEFTextCalc::Bold) ==
         TEFTextCalc::Bold;
  FindCBX(XRCID("ID_COMMENTBOLD"))->SetValue(Set);
  Set = (wxGetApp().KFontStyle & TEFTextCalc::Italic) ==
         TEFTextCalc::Italic;
  FindCBX(XRCID("ID_COMMENTITALIC"))->SetValue(Set);
  Set = (wxGetApp().KFontStyle & TEFTextCalc::Underline) ==
         TEFTextCalc::Underline;
  FindCBX(XRCID("ID_COMMENTUNDERLINE"))->SetValue(Set);
  FindCBX(XRCID("ID_SCROLLTRACK"))->SetValue(wxGetApp().EditFlags &
                                             MyApp::EditScrollTrack);
  FindCBX(XRCID("ID_SW10"))->SetFocus();
  if (wxGetApp().GraphDarst & GraphDataObj::gPunkteVerbinden)
    FindRB(XRCID("ID_INTERPTS"))->SetValue(true);
  else
  if (wxGetApp().GraphDarst & GraphDataObj::gPunkteVerbMark)
    FindRB(XRCID("ID_INTERPTSM"))->SetValue(true);
  else
    FindRB(XRCID("ID_SINGLEPTS"))->SetValue(true);
  FindCBX(XRCID("ID_GRAPHSYMBOL"))->SetValue(~wxGetApp().GraphSymb &
                                             GraphDataObj::GraphOhneSymbol);
  FindCBX(XRCID("ID_GRAPHFETT"))->SetValue(wxGetApp().GraphOpt &
                                           GraphDataObj::GraphFett);
  FindCBX(XRCID("ID_DIAGAVAIL"))->SetValue(wxGetApp().DiagFlags &
                                           MyApp::DiagInNewWorksheet);
  FindCBX(XRCID("ID_DIAGFRAME"))->SetValue(wxGetApp().DiagFlags &
                                           MyApp::DiagHasFrame);
  FindCBX(XRCID("ID_DIAGENDVALUES"))->SetValue(wxGetApp().DiagFlags &
                                               MyApp::DiagEndLabels);
  FindCBX(XRCID("ID_DIAGXVALUESVERT"))->SetValue(wxGetApp().DiagFlags &
                                                 MyApp::DiagXValuesVert);
  FindSC(XRCID("ID_DIAGUNDOCOUNT"))->SetValue(wxGetApp().m_UndoCount);

  // Graph loading format defaults
  FindCBX(XRCID("ID_TAB"))->SetValue(wxGetApp().m_graphdata.TabSeparator);
  FindCBX(XRCID("ID_SPACE"))->SetValue(wxGetApp().m_graphdata.SpaceSeparator);
  FindTC(XRCID("ID_DEC_SEPARATOR"))->SetValue(wxGetApp().m_graphdata.DecSeparator.c_str());
  FindTC(XRCID("ID_LIST_SEPARATOR"))->SetValue(wxGetApp().m_graphdata.ListSeparator.c_str());

  FindCBX(XRCID("ID_GRAPHSORT"))->SetValue(wxGetApp().m_graphdata.Mode &
                                           GraphladenData::GraphSortieren);
  FindCBX(XRCID("ID_GRAPHTEXT"))->SetValue(wxGetApp().m_graphdata.Mode &
                                           GraphladenData::GraphInDatei);
  FindCBX(XRCID("ID_IMG_DATA"))->SetValue(wxGetApp().m_graphdata.ImaginaryDataIncluded);
  FindCBX(XRCID("ID_HEADER"))->SetValue(wxGetApp().m_graphdata.HeaderIncluded);
  FindCBX(XRCID("ID_XVALUES"))->SetValue(wxGetApp().m_graphdata.XValuesIncluded);
  FindSC(XRCID("ID_XCOLUMN"))->SetValue(wxGetApp().m_graphdata.XValuesColumn);
}

TEFOptionenDlg::~TEFOptionenDlg()
{
}

void TEFOptionenDlg::OnReset(wxCommandEvent &)
{
  wxGetApp().GetIniValues(true);
  InitControls();
}

void TEFOptionenDlg::OnOk(wxCommandEvent &)
{
  wxGetApp().ZahlForm = FindCBX(XRCID("ID_SW10"))->GetValue();
  wxGetApp().Options &= ~MyApp::keyLowerCase;
  wxGetApp().Options |= FindCBX(XRCID("ID_SW20"))->GetValue() ?
                        MyApp::keyLowerCase : 0;
  wxGetApp().Options &= ~MyApp::keyWarnSolve;
  wxGetApp().Options |= FindCBX(XRCID("ID_WARN_SOLVE"))->GetValue() ?
                        MyApp::keyWarnSolve : 0;
  wxGetApp().Options &= ~MyApp::keyWarnDelete;
  wxGetApp().Options |= FindCBX(XRCID("ID_WARN_DELETE"))->GetValue() ?
                        MyApp::keyWarnDelete : 0;
  wxGetApp().Options &= ~MyApp::keySaveEMFAsBitmap;
  wxGetApp().Options |= FindCBX(XRCID("ID_EMFFORMAT"))->GetValue() ?
                        MyApp::keySaveEMFAsBitmap : 0;
  wxGetApp().Options &= ~MyApp::keyOnlyRealResults;
  wxGetApp().Options |= FindCBX(XRCID("ID_REALRESULTS"))->GetValue() ?
                        MyApp::keyOnlyRealResults : 0;
  wxGetApp().Options &= ~MyApp::keyFuncSelVisible;
  wxGetApp().Options |= FindCBX(XRCID("ID_FUNCSELVISIBLE"))->GetValue() ?
                        MyApp::keyFuncSelVisible : 0;
  wxGetApp().WinZahlFormat = FindCBX(XRCID("ID_SW21"))->GetValue();
  wxString Zeile = FindTC(XRCID("ID_ERGEBNIS"))->GetValue();
  Zeile.Truncate(199);
  wxGetApp().SetStdFilePath(Zeile);
  wxGetApp().LFontStyle = 0;
  wxGetApp().LFontStyle |= FindCBX(XRCID("ID_LABELBOLD"))->GetValue() ?
                           TEFTextCalc::Bold : 0;
  wxGetApp().LFontStyle |= FindCBX(XRCID("ID_LABELITALIC"))->GetValue() ?
                           TEFTextCalc::Italic : 0;
  wxGetApp().LFontStyle |= FindCBX(XRCID("ID_LABELUNDERLINE"))->GetValue() ?
                           TEFTextCalc::Underline : 0;
  wxGetApp().KFontStyle = 0;
  wxGetApp().KFontStyle |= FindCBX(XRCID("ID_COMMENTBOLD"))->GetValue() ?
                           TEFTextCalc::Bold : 0;
  wxGetApp().KFontStyle |= FindCBX(XRCID("ID_COMMENTITALIC"))->GetValue() ?
                           TEFTextCalc::Italic : 0;
  wxGetApp().KFontStyle |= FindCBX(XRCID("ID_COMMENTUNDERLINE"))->GetValue() ?
                           TEFTextCalc::Underline : 0;
  if (FindRB(XRCID("ID_INTERPTS"))->GetValue())
    wxGetApp().GraphDarst = GraphDataObj::gPunkteVerbinden;
  else
  if (FindRB(XRCID("ID_INTERPTSM"))->GetValue())
    wxGetApp().GraphDarst = GraphDataObj::gPunkteVerbMark;
  else
    wxGetApp().GraphDarst = GraphDataObj::gPunkteMarkieren;
  wxGetApp().DiagFlags = 0;
  wxGetApp().DiagFlags |= FindCBX(XRCID("ID_DIAGAVAIL"))->GetValue() ?
                          MyApp::DiagInNewWorksheet : 0;
  wxGetApp().DiagFlags |= FindCBX(XRCID("ID_DIAGFRAME"))->GetValue() ?
                          MyApp::DiagHasFrame : 0;
  wxGetApp().DiagFlags |= FindCBX(XRCID("ID_DIAGENDVALUES"))->GetValue() ?
                          MyApp::DiagEndLabels : 0;
  wxGetApp().DiagFlags |= FindCBX(XRCID("ID_DIAGXVALUESVERT"))->GetValue() ?
                          MyApp::DiagXValuesVert : 0;
  wxGetApp().GraphSymb &= ~GraphDataObj::GraphOhneSymbol;
  wxGetApp().GraphSymb |= FindCBX(XRCID("ID_GRAPHSYMBOL"))->GetValue() ?
                          GraphDataObj::GraphMitSymbol : GraphDataObj::GraphOhneSymbol;
  wxGetApp().GraphOpt &= ~GraphDataObj::GraphFett;
  wxGetApp().GraphOpt |= FindCBX(XRCID("ID_GRAPHFETT"))->GetValue() ?
                         GraphDataObj::GraphFett : 0;
  wxGetApp().EditFlags = 0;
  wxGetApp().EditFlags |= FindCBX(XRCID("ID_SCROLLTRACK"))->GetValue() ?
                          MyApp::EditScrollTrack : 0;
  wxGetApp().m_UndoCount = FindSC(XRCID("ID_DIAGUNDOCOUNT"))->GetValue();

  // graph loading format defaults
  wxGetApp().m_graphdata.DecSeparator.SetStr(FindTC(XRCID("ID_DEC_SEPARATOR"))->GetValue().c_str());
  wxGetApp().m_graphdata.ListSeparator.SetStr(FindTC(XRCID("ID_LIST_SEPARATOR"))->GetValue().c_str());
  wxGetApp().m_graphdata.TabSeparator = FindCBX(XRCID("ID_TAB"))->GetValue();
  wxGetApp().m_graphdata.SpaceSeparator = FindCBX(XRCID("ID_SPACE"))->GetValue();

  wxGetApp().m_graphdata.Mode = 0;
  wxGetApp().m_graphdata.Mode = (FindCBX(XRCID("ID_GRAPHSORT"))->GetValue() ?
                                 GraphladenData::GraphSortieren : 0) |
                                 (FindCBX(XRCID("ID_GRAPHTEXT"))->GetValue() ?
                                 GraphladenData::GraphInDatei : 0);
  wxGetApp().m_graphdata.ImaginaryDataIncluded = FindCBX(XRCID("ID_IMG_DATA"))->GetValue();
  wxGetApp().m_graphdata.HeaderIncluded = FindCBX(XRCID("ID_HEADER"))->GetValue();
  wxGetApp().m_graphdata.XValuesIncluded = FindCBX(XRCID("ID_XVALUES"))->GetValue();
  wxGetApp().m_graphdata.XValuesColumn = FindSC(XRCID("ID_XCOLUMN"))->GetValue();
  EndModal(wxID_OK);
}

void TEFOptionenDlg::OnLabelFarbe(wxCommandEvent &)
{
  GetColour(this,wxGetApp().LFontColour,_("Label color"));
}

void TEFOptionenDlg::OnDiagColour(wxCommandEvent &)
{
  GetColour(this,wxGetApp().DiagColour,_("Diagram color"));
}

void TEFOptionenDlg::OnCommentFarbe(wxCommandEvent &)
{
  GetColour(this,wxGetApp().KFontColour,_("Comment color"));
}

void TEFOptionenDlg::OnBrowsePath(wxCommandEvent &)
{
  wxDirDialog DirDlg(this,_("Choose the default document path"),
                     FindTC(XRCID("ID_ERGEBNIS"))->GetValue());
  if (DirDlg.ShowModal() == wxID_OK)
    FindTC(XRCID("ID_ERGEBNIS"))->SetValue(DirDlg.GetPath());
}

BEGIN_EVENT_TABLE(TEFOptionenDlg,TEFHilfeDlg)
  EVT_BUTTON(XRCID("ID_RESET"),TEFOptionenDlg::OnReset)
  EVT_BUTTON(wxID_OK,TEFOptionenDlg::OnOk)
  EVT_BUTTON(wxID_CANCEL, TEFOptionenDlg::OnCancel)
  EVT_BUTTON(XRCID("ID_DIAGCOLOUR"), TEFOptionenDlg::OnDiagColour)
  EVT_BUTTON(XRCID("ID_LABELCOLOUR"), TEFOptionenDlg::OnLabelFarbe)
  EVT_BUTTON(XRCID("ID_COMMENTCOLOUR"), TEFOptionenDlg::OnCommentFarbe)
  EVT_BUTTON(XRCID("ID_SELPATH"), TEFOptionenDlg::OnBrowsePath)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// SendControl
// ----------------------------------------------------------------------------

//! Class for static texts that can be send to other windows
class SendControl : public wxStaticText
{
 public:
  //! Ctor
  SendControl(wxWindow *parent,wxWindowID id,const wxString &label)
   : wxStaticText(parent,id,label,wxDefaultPosition,
                  wxDefaultSize,wxALIGN_CENTRE)
  {
    SetForegroundColour(*wxBLUE);
#ifdef __WXMSW__
    wxFont font = GetFont();
    font.SetUnderlined(true);
    SetFont(font);
#endif
  }
  //! Respond to left click by sending event to parent
  void OnLeftClick(wxMouseEvent &)
  {
    wxString tmp = GetLabel();
    wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED,XRCID("CMD_SENDITEM"));
    evt.SetString(GetLabel());
    GetParent()->GetEventHandler()->AddPendingEvent(evt);
  }
 protected:
  DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(SendControl,wxStaticText)
  EVT_LEFT_DOWN(SendControl::OnLeftClick)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// ShowHTMLControl
// ----------------------------------------------------------------------------

//! Class for showing HTML files on mouseclick
class ShowHTMLControl : public wxStaticText
{
 protected:
   wxString m_htmlfile;
 public:
  //! Ctor
  ShowHTMLControl(wxWindow *parent,wxWindowID id,const wxString &label, const wxString &file)
   : wxStaticText(parent,id,label,wxDefaultPosition,
                  wxDefaultSize,wxALIGN_CENTRE)
  {
    SetForegroundColour(*wxBLUE);
#ifdef __WXMSW__
    wxFont font = GetFont();
    font.SetUnderlined(true);
    SetFont(font);
    m_htmlfile = file;
#endif
  }
  //! Respond to left click by showing the file
  void OnLeftClick(wxMouseEvent &)
  {
    wxFrame *frame = new wxFrame(GetParent(),-1,_("Licence"),wxDefaultPosition,wxSize(640,480));
    wxHtmlWindow *wnd = new wxHtmlWindow(frame,-1);
    wnd->LoadFile(m_htmlfile);
    frame->Centre();
    frame->Show();
  }
 protected:
  DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(ShowHTMLControl,wxStaticText)
  EVT_LEFT_DOWN(ShowHTMLControl::OnLeftClick)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFAboutDlg
// ----------------------------------------------------------------------------

void TEFAboutDlg::OnSendItem(wxCommandEvent &event)
{
  wxString title = wxT("mailto:markusgreither@yahoo.de?subject=");
  title += _("User feedback Easy Funktion ");
  title += wxT(VERSIONSTRING);
  if (!wxLaunchDefaultBrowser(title))
    wxMessageBox(_("Couldn't send eMail!"));
  EndModal(wxID_OK);
}

void TEFAboutDlg::Create(wxWindow *parent,wxWindowID Id,const wxString &title)
{
  wxXmlResource::Get()->LoadDialog(this, parent, wxT("dlgAbout"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);

  wxString Z = wxT("Version ");
  Z += wxT(VERSIONSTRING);
  FindST(XRCID("ID_VERSION"))->SetLabel(Z);

  Z = wxT(COPYRIGHTSTRING);
  FindST(XRCID("ID_COPYRIGHT"))->SetLabel(Z);

  wxString file;
  wxGetApp().GetAppDataDir(file);
  file += wxT("/gpl.html");

  ShowHTMLControl *gpl = new ShowHTMLControl(this,-1,
                                       _("Show GPL licence"),file);
  wxXmlResource::Get()->AttachUnknownControl(wxT("ID_LICENCE"),gpl);
  SendControl *EMail = new SendControl(this,-1,
                                       wxT("mailto:markusgreither@yahoo.de"));
  wxXmlResource::Get()->AttachUnknownControl(wxT("ID_EMAILCONTROL"),EMail);
  wxHyperlinkCtrl *wxHome = new wxHyperlinkCtrl(this,-1,
                                                wxT("Homepage"),
                                                wxT("http://easyfunktion.sourceforge.net/"),
                                                wxDefaultPosition,
                                                wxDefaultSize,
                                                wxALIGN_CENTRE | wxHL_DEFAULT_STYLE);
  wxXmlResource::Get()->AttachUnknownControl(wxT("ID_HOMEPAGE"),wxHome);
  wxHyperlinkCtrl *wxWindows = new wxHyperlinkCtrl(this,-1,
                                                   wxT("http://www.wxwidgets.org"),
			                           wxT("http://www.wxwidgets.org"),
			                           wxDefaultPosition,
			                           wxDefaultSize,
			                           wxALIGN_CENTRE | wxHL_DEFAULT_STYLE);
  wxXmlResource::Get()->AttachUnknownControl(wxT("ID_WXCONTROL"),wxWindows);
  Fit();
  Centre();
}

BEGIN_EVENT_TABLE(TEFAboutDlg,TEFHilfeDlg)
  EVT_MENU(XRCID("CMD_SENDITEM"), TEFAboutDlg::OnSendItem)
  EVT_BUTTON(wxID_OK,TEFAboutDlg::OnOK)
END_EVENT_TABLE()


