// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efdocum.cpp
    \brief Source code for the document class of the Doc/View framework
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>

#include <wx/clipbrd.h>

#include <complex>
#include <limits>
#include <climits>
#include <fstream>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "gauss.h"
#include "efgemein.h"
#include "efsolver.h"
#include "ef.h"
#include "efedit.h"
#include "efeditsr.h"
#include "efdocum.h"
#include "efresid.h"
#include "eftabcon.h"
#include "efgraphc.h"
#include "efview.h"

#endif

#include <wx/txtstrm.h>

#ifdef __WXMSW__
    #include "wx/msw/mslu.h"
#endif

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

// ----------------------------------------------------------------------------
// TEFDocument
// ----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(TEFDocument,wxDocument)

TEFDocument::TEFDocument()
 : EFDaten(wxGetApp().GetLDV()),m_EmptyFile(false)
{
  if (wxGetApp().Options & MyApp::keyOnlyRealResults)
    m_Parser = new ParserDoubleObj(wxGetApp().GetLDV());
  else
    m_Parser = new ParserCpxObj(wxGetApp().GetLDV());
}

TEFDocument::~TEFDocument()
{
  delete m_Parser;
}

void TEFDocument::StdPostTerm(DiagDataPtArrObj &ar)
{
  ZahlFormatObj ZF(m_Parser->GetLDV());

  ar.Begin();
  if (ar.GetMaxEintrag() >= 0)
    do
      if (ar.GetAkEint()->PostProcTerm[0] != 0)
      {
        MGPStr Zeile(ar.GetAkEint()->PostProcTerm.c_str());
        ZF.WinToStd(Zeile);
        ar.GetAkEint()->PostProcTerm.SetStr(Zeile.c_str());
      }
    while (ar.NextEint());
}

void TEFDocument::WinPostTerm(DiagDataPtArrObj &ar)
{
  MGPStr MGZeile;
  ZahlFormatObj ZF(m_Parser->GetLDV());

  ar.Begin();
  if (ar.GetMaxEintrag() >= 0)
    do
      if (ar.GetAkEint()->PostProcTerm[0] != 0)
      {
        MGPStr Zeile(ar.GetAkEint()->PostProcTerm.c_str());
        ZF.StdToWin(Zeile);
        ar.GetAkEint()->PostProcTerm.SetStr(Zeile.c_str());
      }
    while (ar.NextEint());
}

void TEFDocument::WinPostTerm(DiagDataArrObj &ar)
{
  MGPStr MGZeile;
  ZahlFormatObj ZF(m_Parser->GetLDV());

  ar.Begin();
  if (ar.GetMaxEintrag() >= 0)
    do
      if (ar.GetAkEint().PostProcTerm[0] != 0)
      {
        MGPStr Zeile(ar.GetAkEint().PostProcTerm.c_str());
        ZF.StdToWin(Zeile);
        ar.GetAkEint().PostProcTerm.SetStr(Zeile.c_str());
      }
    while (ar.NextEint());
}

// AkStr in Zeilen zerteilen und Win->Std bzw. Std->Win konvertieren
void TEFDocument::ConvertLines(MGPStr &AkStr,ZahlFormatObj &ZF,bool ToWin)
{
  MGPStr AkZ,AlleZ;
  const _TCHAR *tmp = AkStr.c_str();
  do
  {
    const _TCHAR *tmp1 = _tcsstr(tmp,wxT("\r\n"));
    AkZ.SetStr(wxT(""));
    if (tmp1)
      AkZ.AddNChar(tmp,tmp1-tmp);
    else
      AkZ.AddStr(tmp);
    if (ToWin)
      ZF.StdToWin(AkZ);
    else
      ZF.WinToStd(AkZ);
    AlleZ.AddStr(AkZ);
    if (tmp1)
    {
      AlleZ.AddStr(wxT("\r\n"));
      tmp = tmp1+2;
    }
    else
      tmp = 0;
  }
  while (tmp);
  AkStr.SetStr(AlleZ);
}

void TEFDocument::StdEquations()
{
  ZahlFormatObj ZF(m_Parser->GetLDV());

  EquationsArr->Begin();
  while (EquationsArr->NextEint())
    if (EquationsArr->AkEquations()->Equations[0] != 0)
      ConvertLines(EquationsArr->AkEquations()->Equations,ZF,false);
}

void TEFDocument::WinEquations()
{
  ZahlFormatObj ZF(m_Parser->GetLDV());

  EquationsArr->Begin();
  while (EquationsArr->NextEint())
    if (EquationsArr->AkEquations()->Equations[0] != 0)
      ConvertLines(EquationsArr->AkEquations()->Equations,ZF,true);
}

void TEFDocument::StdFunk()
{
  MGPStr MGZeile;
  int p = Funktion->AkEintNummer();
  ZahlFormatObj ZF(m_Parser->GetLDV());

  Funktion->Begin();
  while (Funktion->NextEint())
  {
    if ((*Funktion)->Term[0] != 0)
    {
      MGZeile.SetStr((*Funktion)->Term);
      ZF.WinToStd(MGZeile);
      (*Funktion)->Term.SetStr(MGZeile);
      MGZeile.SetStr((*Funktion)->ParamStr);
      ZF.WinToStd(MGZeile);
      (*Funktion)->ParamStr.SetStr(MGZeile);
    }
  }
  Funktion->SetAkEint(p);
}

void TEFDocument::WinFunk()
{
  MGPStr MGZeile;
  int p = Funktion->AkEintNummer();
  ZahlFormatObj ZF(m_Parser->GetLDV());

  Funktion->Begin();
  while (Funktion->NextEint())
  {
    if ((*Funktion)->Term[0] != 0)
    {
      MGZeile.SetStr((*Funktion)->Term);
      ZF.StdToWin(MGZeile);
      (*Funktion)->Term.SetStr(MGZeile);
      MGZeile.SetStr((*Funktion)->ParamStr);
      ZF.StdToWin(MGZeile);
      (*Funktion)->ParamStr.SetStr(MGZeile);
    }
  }
  Funktion->SetAkEint(p);
}

bool TEFDocument::OnSaveDocument(const wxString& file)
{ // ueberladen wg ios::binary-mode
  if ( !file )
      return false;

  if (!SaveFile(file))
      return false;

  m_savedYet = true;

  Modify(false);
  SetFilename(file);
  return true;
}

bool TEFDocument::OnOpenDocument(const wxString& file)
{ // ueberladen wg ios_base::binary-mode
  if (!OnSaveModified())
      return false;

  if (!LoadFile(file))
      return false;

  SetFilename(file, true);
  Modify(FALSE);
  m_savedYet = true;

  UpdateAllViews();

  return true;
}

bool TEFDocument::LoadFile(const wxString &file)
{
  ifstream store(file.mb_str(),ios::binary | ios::in);
  wxString msgTitle;
  if (wxTheApp->GetAppName() != wxT(""))
      msgTitle = wxTheApp->GetAppName();
  else
      msgTitle = wxString(_("File error"));
  if (store.fail() || store.bad())
  {
      wxString tmp = _("Error opening file ") + file;
      wxMessageBox(tmp, msgTitle, wxOK|wxICON_EXCLAMATION, GetDocumentWindow());
      return false;
  }
  wxBeginBusyCursor();
  bool erg;
  if (store.peek() == '<')
    erg = XmlDateiLaden(store); // Load file in XML format
  else
#ifdef __MINGW32__
  if (store.peek() == EOF)
#else
  if (store.peek() == char_traits<char>::eof())
#endif
  {
    erg = true; // accept empty files created by the shell
    m_EmptyFile = true;
  }
  else
    erg = EinstDateiLaden(store); // Load file in old text format
  WinFunk(); // Funktionen in Windows-Format wandeln
  WinEquations(); // Gleichungen in Windows-Format wandeln
  m_Parser->StdToWin(NaehData.ParamStr);
  WinPostTerm(*DiagDataArr);
  wxEndBusyCursor();
  if (!erg)
  {
      wxString tmp = _("Error reading file %s") + file;
      wxMessageBox(tmp, msgTitle, wxOK|wxICON_EXCLAMATION, GetDocumentWindow());
      return false;
  }

  return true;
}

bool TEFDocument::SaveFile(const wxString &file)
{
  wxString msgTitle;
  if (wxTheApp->GetAppName() != wxT(""))
      msgTitle = wxTheApp->GetAppName();
  else
      msgTitle = wxString(_("File error"));
  ofstream store(file.mb_str(),ios::binary | ios::out);
  if (store.fail() || store.bad())
  {
      (void)wxMessageBox(_("Sorry, could not open this file for saving."),
                         msgTitle, wxOK | wxICON_EXCLAMATION,
                         GetDocumentWindow());
      // Saving error
      return false;
  }
  wxBeginBusyCursor();
  TEFView *view = (TEFView *)GetFirstView();
  PictureArr->ResetKette(); // delete old copies
  DiagDataArr->ResetKette();
  EquationsArr->ResetKette();
  DiagDataPtArrObj DiagDataPts;
  view->VnCommit(DiagDataPts);
  StdFunk(); // Funktionen in Englisches Format wandeln
  StdEquations(); // Gleichungen in Englisches Format wandeln
  MGPStr tmp = NaehData.ParamStr;
  // Delete ParamStr if loaded from graph to save disk space
  if (NaehData.m_CurGraph)
    NaehData.ParamStr.SetStr(wxT(""));
  else
    m_Parser->WinToStd(NaehData.ParamStr);
  StdPostTerm(DiagDataPts);
  bool erg = XmlDateiSpeichern(store,DiagDataPts);
  if (!erg)
  {
      (void)wxMessageBox(_("Sorry, could not save this file."),
                         msgTitle, wxOK | wxICON_EXCLAMATION,
                         GetDocumentWindow());
      // Saving error
      return false;
  }
  PictureArr->ResetKette(); // delete temporary copies
  DiagDataArr->ResetKette();
  EquationsArr->ResetKette();
  WinFunk(); // Funktionen in Windows-Format zurueckwandeln
  WinEquations();
  NaehData.ParamStr = tmp;
  WinPostTerm(DiagDataPts);
  wxEndBusyCursor();

  return true;
}

bool TEFDocument::IsModified() const
{
  TEFView *view = (TEFView *)GetFirstView();
  return wxDocument::IsModified() | (view ? view->IsModified() : false);
}

void TEFDocument::Modify(bool mod)
{
  wxDocument::Modify(mod);
  TEFView *view = (TEFView *)GetFirstView();
  if (view && !mod)
    view->ClearModify();
}


