// Version 6.06  29.11.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efeditsr.cpp
    \brief Source code for the function editor with search/replace functionality
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h>

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>

#include <wx/clipbrd.h>
#include <wx/fdrepdlg.h>

#include <complex>
#include <limits>
#include <cstdio>
#include <cstring>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "efresid.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "efedit.h"
#include "efeditsr.h"
#include "ef.h"

#endif

// ----------------------------------------------------------------------------
// TEFEditSearch
// ----------------------------------------------------------------------------

BEGIN_EVENT_TABLE(TEFEditSearch, TEFEdit)
  EVT_RIGHT_UP(TEFEditSearch::OnContextMenu)
  EVT_LEFT_DOWN(TEFEditSearch::OnLButtonDown)
  EVT_KEY_DOWN(TEFEditSearch::OnKeyDown)
  EVT_KILL_FOCUS(TEFEditSearch::OnKillFocus)
  EVT_SET_FOCUS(TEFEditSearch::OnSetFocus)
  EVT_UPDATE_UI(wxID_FIND,TEFEditSearch::CeEditFindReplace)
  EVT_UPDATE_UI(XRCID("CMD_EDITREPLACE"),TEFEditSearch::CeEditFindReplace)
  EVT_MENU(wxID_FIND, TEFEditSearch::OnEditFind)
  EVT_MENU(XRCID("CMD_EDITREPLACE"), TEFEditSearch::OnEditReplace)
  EVT_FIND_REPLACE(-1,TEFEditSearch::OnFindReplace)
  EVT_FIND_REPLACE_ALL(-1,TEFEditSearch::OnFindReplaceAll)
  EVT_FIND(-1,TEFEditSearch::OnEditFindNext)
  EVT_FIND_NEXT(-1,TEFEditSearch::OnEditFindNext)
  EVT_FIND_CLOSE(-1,TEFEditSearch::OnFindClose)
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(TEFEditSearch,TEFEdit)

//
// Construct a TEFEditSearch window given some initial text.
//
TEFEditSearch::TEFEditSearch(wxWindow* parent,
                             wxWindowID id,
                             const wxPoint &pos,
                             const wxSize &size,
                             long style,
                             const wxString &Title,
                             const NumListFormat &LDV,
                             EditMode DialMode)
:
  TEFEdit(parent,id,pos,size,style,Title,LDV,DialMode)
  ,SearchData(wxFR_DOWN)
{
  SearchDialog = 0;
}

TEFEditSearch::~TEFEditSearch()
{
  delete SearchDialog;
}

void TEFEditSearch::OnContextMenu(wxMouseEvent &event)
{ // gibt Context-Menue aus
  wxPoint hp;
  hp = event.GetPosition();
  if (FindFocus() == this)
  {
    wxMenu *PMenue = wxGetApp().CreateCalcMenu(true);
    PopupMenu(PMenue,hp);
    delete PMenue;
  }
  else
    event.Skip();
}

void TEFEditSearch::ShowCaretPos()
{ // Zeigt aktuelle Zeilennummer und Zeilenposition an
  unsigned int Anf,End,LnNmb;
  MGPStr MGZeile;

  GetSelection(Anf,End);
  LnNmb = GetLineFromPos(Anf);
  MGZeile.AddInt(LnNmb+1);
  MGZeile.AddStr(wxGetApp().GetLDV().ListSeparator);
  MGZeile.AddInt(Anf-GetLineIndex(LnNmb));
  wxGetApp().sb->SetStatusText(MGZeile.c_str(),1);
}

bool TEFEditSearch::SetSelection(unsigned int Anf,unsigned int End,bool Scroll)
{ // Aktualisiert die Zeilenposition in der Statuszeile
  bool ret;

  ret = TEFEdit::SetSelection(Anf,End,Scroll);
  ShowCaretPos();
  return ret;
}

void TEFEditSearch::OnLButtonDown(wxMouseEvent &event)
{ // Aktualisiert die Zeilenposition in der Statuszeile
  TEFEdit::OnLButtonDown(event);
  ShowCaretPos();
}

void TEFEditSearch::OnKeyDown(wxKeyEvent &event)
{ // Aktualisiert die Zeilenposition in der Statuszeile
  TEFEdit::OnKeyDown(event);
  ShowCaretPos();
}

void TEFEditSearch::OnSetFocus(wxFocusEvent &event)
{ // Erzeugt Zeilenposition in Statuszeile
//  TEFEdit::OnSetFocus(event);
  ShowCaretPos();
  event.Skip();
}

void TEFEditSearch::OnKillFocus(wxFocusEvent &event)
{ // loescht Zeilenposition in Statuszeile
//  TEFEdit::OnKillFocus(event);
  wxGetApp().sb->SetStatusText(wxT(""),1);
  event.Skip();
}


//
// Perform a search or replace operation based on information in SearchData
//
void
TEFEditSearch::DoSearch(ReplaceMode Mode)
{
  unsigned int AnfPos,EndPos;
  do
  {
    if (Mode == Replace)
    {
      GetSelection(AnfPos,EndPos);
      SetSelection(AnfPos,AnfPos);
    }
    if (Search(-1, SearchData.GetFindString().c_str(),
               bool(SearchData.GetFlags() & wxFR_MATCHCASE),
               bool(SearchData.GetFlags() & wxFR_WHOLEWORD),
               !(SearchData.GetFlags() & wxFR_DOWN)) >= 0)
    {
      if ((Mode == Replace) || (Mode == ReplaceAll))
        Insert(SearchData.GetReplaceString());
    }
    else
    {
      if (Mode != ReplaceAll)
      {
        _TCHAR errMsg[200];
        wxString str(_("Cannot find string '%s' !"));
        _stprintf(errMsg, str.EF_C_STR(),
                SearchData.GetFindString().EF_C_STR());
        wxWindow* w = SearchDialog ? (wxWindow*)SearchDialog : (wxWindow*)this;
        wxMessageBox(errMsg, wxT("Easy Funktion"),
                      wxOK | wxICON_EXCLAMATION,w);
      }
      else
      if (Mode == ReplaceAll)
        break;
    }
  } 
  while (Mode == ReplaceAll);
}

//
// Open the modeless Find commdlg
//

void TEFEditSearch::OnEditFind(wxCommandEvent&)
{
  EditFindReplace(Find);
}

void TEFEditSearch::OnEditFindNext(wxFindDialogEvent&)
{
  EditFindReplace(Find);
}

void TEFEditSearch::OnFindClose(wxFindDialogEvent&)
{
  SearchDialog->Destroy();
  SearchDialog = 0;
}

void TEFEditSearch::OnFindReplace(wxFindDialogEvent&)
{
  EditFindReplace(Replace);
}

void TEFEditSearch::OnFindReplaceAll(wxFindDialogEvent&)
{
  EditFindReplace(ReplaceAll);
}

void TEFEditSearch::EditFindReplace(ReplaceMode Mode)
{
  if (SearchDialog)
  {
    DoSearch(Mode);
  }
  else
  {
    wxString title = (Mode == Replace ? _("Replace Text") : _("Search Text"));
    SearchDialog = new wxFindReplaceDialog(this, &SearchData,title,
                                           Mode == Replace ?
                                           wxFR_REPLACEDIALOG : 0);
    SearchDialog->Show(true);
  }
}

//
// Open the modeless Replace commdlg
//
void
TEFEditSearch::OnEditReplace(wxCommandEvent&)
{
  EditFindReplace(Replace);
}

//
// Enable the find or replace option only if no dialog is up
//
void
TEFEditSearch::CeEditFindReplace(wxUpdateUIEvent& ce)
{
  ce.Enable((FindFocus() == this) && !SearchDialog);
}

