// Version 6.06  29.11.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efgrcmds.cpp
    \brief Source for the diagram window class event handling
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h>

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/resizec.h>

#include <wx/clipbrd.h>
#include <wx/metafile.h>

#include <cfloat>
#include <cstring>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif
#include <complex>
#include <limits>
#include <cstdio>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"
#include "efmcxio.h"
#include "eftabcon.h"
#include "efgraphc.h"
#include "efedit.h"
#include "efresid.h"
#include "efgrdlgs.h"

#endif

#ifdef __WXMSW__
#include <windows.h>
#include <wx/msw/winundef.h>
#endif

#include "efmacro.h"
#include "efgrdnd.h" // local include because of problem with prec. headers

// ----------------------------------------------------------------------------
// TEFGraphControl
// ----------------------------------------------------------------------------

void TEFGraphControl::SendSetAkDiag()
{
  wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,
                       XRCID("ICM_SETAKDIAG"));
  event.SetEventObject(this);
  GetParent()->GetEventHandler()->ProcessEvent(event);
}

void TEFGraphControl::SendDiagModified()
{
  wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,
                       XRCID("ICM_SETDIAGMODIFIED"));
  event.SetEventObject(this);
  GetParent()->GetEventHandler()->ProcessEvent(event);
}

void TEFGraphControl::SendModified()
{
  wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,
                       XRCID("ICM_SETMODIFIED"));
  event.SetEventObject(this);
  GetParent()->GetEventHandler()->ProcessEvent(event);
}

void TEFGraphControl::SendGetFunktions()
{
  wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_GETFUNKTIONS"));
  event.SetEventObject(this);
  GetParent()->GetEventHandler()->ProcessEvent(event);
}

void TEFGraphControl::OnMoveLegend(wxCommandEvent &)
{
  m_curpos = wxGetMousePosition();
  m_curpos = ScreenToClient(m_curpos);
  m_lastcurpos = m_curpos;
  m_moved = false;
  // Workaround for broken event.IsChecked() from PopupMenu
  // Use m_capt instead
  if (!m_capt)
  {
    m_movemode = Legende;
    CaptureMouse();
    m_capt = true;
  }
  else
  {
    if (HasCapture())
      ReleaseMouse();
    m_movemode = MoveWin;
    m_capt = false;
  }
  Invalidate();
}

void TEFGraphControl::OnMoveLabel(wxCommandEvent &)
{
  m_curpos = wxGetMousePosition();
  m_curpos = ScreenToClient(m_curpos);
  m_lastcurpos = m_curpos;
  m_moved = false;
  // Workaround for broken event.IsChecked() from PopupMenu
  // Use m_capt instead
  if (!m_capt)
  {
    m_movemode = Label;
    CaptureMouse();
    m_capt = true;
  }
  else
  {
    if (HasCapture())
      ReleaseMouse();
    m_movemode = MoveWin;
    m_capt = false;
  }
  Invalidate();
}

void TEFGraphControl::OnEditUndo(wxCommandEvent &)
{
  UndoRedo(false);
}

void TEFGraphControl::OnEditRedo(wxCommandEvent &)
{
  UndoRedo(true);
}

void TEFGraphControl::CeEditUndo(wxUpdateUIEvent& ce)
{
  ce.Enable((FindFocus() == this) && (m_UndoCount > 0));
}

void TEFGraphControl::CeEditRedo(wxUpdateUIEvent& ce)
{
  ce.Enable((FindFocus() == this) && (m_UndoCount <= m_UndoItems.GetMaxEintrag()));
}

void TEFGraphControl::OnKillFocus(wxFocusEvent &event)
{
  if (m_hasfocus)
  {
    wxClientDC DC(this);
    m_hasfocus = false;
    DrawSizeRect(DC);
    SetScreenParams(DC);
    FocusRect(DC);
    GetAkDiag().Text(true);
    DiagCurPos(DC);
    Diag1.Mark1.UnMark(DC);
    Diag2.Mark1.UnMark(DC);
  }
  event.Skip();
}

void TEFGraphControl::OnSetFocus(wxFocusEvent &event)
{
  if (!m_hasfocus)
  {
    Raise();
    wxClientDC DC(this);
    SetScreenParams(DC);
    DC.SetFont(*LegendeFont);
    InitGraphs(DDiagData.AkGraph,DC);
    DC.SetFont(wxNullFont);
    if (GetAkDiag().Mark1.Markiert() || GetAkDiag().Mark2.Markiert())
      DiagCurPos(DC);
    SendSetAkDiag();
    // m_hasfocus erst ab hier true, wg. DrawSizeRect() in SetActive() !
    m_hasfocus = true;
    FocusRect(DC);
    DC.SetUserScale(1.0,1.0);
    DrawSizeRect(DC);
  }
  event.Skip();
}

void TEFGraphControl::OnContextMenu(wxMouseEvent &event)
{
  wxPoint hp;
  hp = event.GetPosition();
  if (m_capt)
  { // Release capture if Label move or Legend move active
    if (HasCapture())
      ReleaseMouse();
    m_capt = false;
  }
  if (FunkMenue.HoleBefehl(hp.x,hp.y) >= 0)
  { // Legend under Cursor
    wxClientDC DC(this);
    SetScreenParams(DC);
    SelectGraphFromPos(DC,hp);
    wxMenu *PMenue = wxGetApp().CreateDiagramMenu(false);
    wxMenu *menu;
    if (PMenue->FindItem(XRCID("CMD_DELETEGRAPH"),&menu) == 0)
    {
      wxMessageBox(_("Graph Menu not found !"));
      return;
    }
    // Need to create a copy of Submenu as using it directly in PopupMenu() doesn't work on wx 2.9.1 (??!!)
    wxMenu *tmpm = new wxMenu;
    for (int i = 0; i < menu->GetMenuItemCount();i++)
    {
      wxMenuItem *mi = menu->FindItemByPosition(i);
      tmpm->Append(mi->GetId(),mi->GetItemLabel(),mi->GetHelp(),mi->GetKind());
    }
    delete PMenue;
    MenuCursor = true;
    PopupMenu(tmpm,hp);
    delete tmpm;
    MenuCursor = false;
  }
  else
  if (SetActiveLabel(FindLabel(hp.x,hp.y)) && (FindFocus() == this))
  { // Label under Cursor
    wxMenu *menu = wxXmlResource::Get()->LoadMenu(wxT("menuLabel"));
    MenuCursor = true;
    PopupMenu(menu,hp);
    MenuCursor = false;
    delete menu;
  }
  else
  if (FindFocus() == this)
  {
    wxClientDC DC(this);
    SetScreenParams(DC);
    MarkDiag(hp,DC,true);
    wxMenu *PMenue = wxGetApp().CreateDiagramMenu(true);
    MenuCursor = true;
    // Remove diagram flags menu because of broken IsChecked() after PopupMenu
    PMenue->Destroy(XRCID("CMD_DIAGPROPMENU"));
    PMenue->Destroy(XRCID("CMD_MARKMENU"));
    PopupMenu(PMenue,hp);
    MenuCursor = false;
    delete PMenue;
  }
  else
    event.Skip();
  if (DelDiag)
  { // delayed delete of diagram, so that PopupMenu surely finished
    DelDiag = false; // Prevent problems after evtl. veto
    wxCommandEvent cevent;
    OnDelDiag(cevent);
  }
}

void TEFGraphControl::OnLButtonDblClk(wxMouseEvent &event)
{
  int a1;
  wxPoint hp;
  hp.x = event.GetX();
  hp.y = event.GetY();
  ScreenToClient(hp);
  a1 = FunkMenue.HoleBefehl(hp.x,hp.y);
  wxCommandEvent cevent;
  if (m_capt)
  {
    ReleaseMouse();
    m_capt = false;
  }
  if (a1 >= 0)
    OnGraphOptionen(cevent); // Punkt in Graph-MenueItem
  else
  if (SetActiveLabel(FindLabel(hp.x,hp.y)))
    LabelOptions(*AkLabel);
  else
  if (Diag1.PunktInDiagramm(hp.x,hp.y)) // Punkt im Diagramm 1
    DarstellungsDialog(Diag1.PunktInLabel(hp.x,hp.y) ? 1 : 0,
                       Diag1.GetYAxisNumber(hp.x,hp.y));
  else
  if (ZweiDiags())
  {
    if (Diag2.PunktInDiagramm(hp.x,hp.y)) // Punkt im Diagramm 2
      DarstellungsDialog(Diag2.PunktInLabel(hp.x,hp.y) ? 1 : 0,
                         Diag2.GetYAxisNumber(hp.x,hp.y));
  }
}

void TEFGraphControl::OnLButtonDown(wxMouseEvent &event)
{
  if (m_capt)
    OnLButtonUp(event);

  wxClientDC DC(this);
  SetScreenParams(DC);
  wxPoint hp = event.GetPosition();
  ScreenToClient(hp);
  m_curpos = hp;
  m_lastcurpos = hp;
  if (!m_hasfocus)
  {
    // im Augenblick (wxWin 2.2.1) nie aufgerufen,
    // weil WM_LBUTTONDOWN in wxWindow SetFocus() aufruft !
    SetFocus();
    m_hasfocus = true;
    m_movemode = MoveWin;
  }
  else
  {
    m_movemode = PointInSizeRect(hp);
    if (m_movemode == MoveWin)
    {
      int a = FunkMenue.HoleBefehl(hp.x,hp.y);
      if (a >= 0)
      {
        m_movemode = GraphSelect; // MenueItem in Legende angeklickt
        SelectGraphFromPos(DC,hp);
      }
      else
      if (a == -2)
        m_movemode = Legende; // Legende angeklickt
      else
      if (SetActiveLabel(FindLabel(hp.x,hp.y)))
        m_movemode = Label; // Label angeklickt
      else
      if (MarkDiag(hp,DC,true) && GetAkDiag().PunktInObject(hp.x,hp.y))
      {
        MarkDiag(hp,DC,false);
        m_movemode = Graph; // Diagramm angeklickt
      }
      if ((m_movemode == Graph) && wxGetApp().GetZoomIn())
        m_movemode = ZoomIn;
    }
  }
  CaptureMouse();
  m_capt = true;
  m_moved = false;
}

void TEFGraphControl::OnMouseMove(wxMouseEvent &event)
{
  wxPoint hp = event.GetPosition();
  ScreenToClient(hp);
  if (hp != m_lastcurpos)
  {
    if (m_capt)
    {
      if (m_movemode == Label)
      {
        MoveLabel(hp,false);
      }
      else
      if (m_movemode == GraphSelect)
      {
        m_moved |= ((abs(m_curpos.x-hp.x) > 10) || (abs(m_curpos.y-hp.y) > 10));
        if (m_moved)
        {
          ReleaseMouse();
          wxCustomDataObject *doo = GetGraphData();
          wxDropSource dragsrc(this);
          dragsrc.SetData(*doo);
          dragsrc.DoDragDrop(true);
          m_capt = false;
          return;
        }
      }
      else
      if (m_movemode == ZoomIn)
      {
        wxClientDC DC(this);
        wxRect rect(m_curpos,
                    wxSize(m_lastcurpos.x-m_curpos.x,m_lastcurpos.y-m_curpos.y));
        if (m_moved)
          DrawFocusRect(DC,rect);
        m_moved |= ((abs(m_curpos.x-hp.x) > 10) || (abs(m_curpos.y-hp.y) > 10));

        wxRect rect1(m_curpos,wxSize(hp.x-m_curpos.x,hp.y-m_curpos.y));
        if (m_moved)
          DrawFocusRect(DC,rect1);
      }
      if (m_movemode == Graph)
      {
        wxClientDC DC(this);
        SetScreenParams(DC);
        wxPoint tmp;
        tmp.x = event.GetX();
        tmp.y = event.GetY();
        ScreenToClient(tmp);
        MarkDiag(tmp,DC,false);
      }
      else
      if (m_movemode == Legende)
      {
        MoveLegend(hp,false);
      }
      else
      {
        if (m_moved)
          DrawMoveRect(m_lastcurpos,m_movemode,GetRatio());
        m_moved |= ((abs(m_curpos.x-hp.x) > 10) || (abs(m_curpos.y-hp.y) > 10));
        if (m_moved)
          DrawMoveRect(hp,m_movemode,GetRatio());
      }
    }
    m_lastcurpos = hp;
    if (FindFocus() == this)
    { // wenn dieses Fenster den Focus hat, Cursorposition ausgeben
      wxClientDC DC(this);
      SetScreenParams(DC);
      EFDiagObj *InDiag = 0;
      if (Diag1.PunktInObject(hp.x,hp.y)) // Position in Diag1
        InDiag = &Diag1;
      else
      if (ZweiDiags() && Diag2.PunktInObject(hp.x,hp.y))
        InDiag = &Diag2;
      if (InDiag)
      { // wenn Mouseposition in einem Diagramm, Position ausgeben
        MGPStr tmp;
        MGPStr Zeile;
        tmp.SetStr(wxT(" X:"));
        Parser.Ausgabe(Zeile,InDiag->XWert(hp.x),wxGetApp().ZahlForm);
        tmp.AddStr(Zeile);
        tmp.AddStr(Parser.GetLDV().ListSeparator);
        tmp.AddStr(wxT(" Y:"));
        Parser.Ausgabe(Zeile,InDiag->YWert(hp.y),wxGetApp().ZahlForm);
        tmp.AddStr(Zeile);
        wxGetApp().sb->SetStatusText(wxString(tmp.c_str()),1);
      }
      else
        wxGetApp().sb->SetStatusText(wxString(wxT("")),1);
    }
  }
#ifdef __WXGTK__
  wxSetCursorEvent evt(hp.x,hp.y);
  OnSetCursor(evt);
#endif
}

void TEFGraphControl::MoveLabel(const wxPoint &hp,bool finished,bool domove)
{
  if (finished)
    AddUndo(UndoDiagDataBase);
  wxClientDC DC(this);
  wxPoint p1 = GetLabelPos(*AkLabel);
  wxSize sz = GetLabelSize(*AkLabel);
  wxPoint p2 = p1;
  p2.x += sz.x;
  p2.y += sz.y;
  wxRect tmp(p1,p2);
  if (!finished)
  {
    if (m_moved)
    {
      wxRect tmp1(tmp);
      tmp1.x += m_lastcurpos.x-m_curpos.x;
      tmp1.y += m_lastcurpos.y-m_curpos.y;
      DrawFocusRect(DC,tmp1);
    }
    m_moved |= ((abs(m_curpos.x-hp.x) > 10) || (abs(m_curpos.y-hp.y) > 10));
    if (m_moved)
    {
      wxRect tmp1(tmp);
      tmp1.x += hp.x-m_curpos.x;
      tmp1.y += hp.y-m_curpos.y;
      DrawFocusRect(DC,tmp1);
    }
  }
  else
  {
    wxPoint delta(m_lastcurpos.x-m_curpos.x,m_lastcurpos.y-m_curpos.y);
    tmp.x += m_lastcurpos.x-m_curpos.x;
    tmp.y += m_lastcurpos.y-m_curpos.y;
    if (m_moved)
      DrawFocusRect(DC,tmp);
    if (domove)
    {
      LabelArrObj tmpbase(DDiagData.Labels);
      MoveLabel(*AkLabel,delta.x,delta.y);
      if (MyMacroRecorder::Get()->IsRecording())
      {
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_MODIFYLABEL"),GetId());
        EFXmlNode *node = MyMacroRecorder::Get()->GetAkNode();
        DDiagData.Labels.XmlWrite(node,&tmpbase);
      }
      Invalidate();
      SendModified();
    }
  }
}

void TEFGraphControl::MoveLegend(const wxPoint &hp,bool finished,bool domove)
{
  if (finished)
    AddUndo(UndoDiagDataBase);
  wxClientDC DC(this);
  wxPoint p1(FunkMenue.BildBer.XMin,FunkMenue.BildBer.YMin);
  wxPoint p2(FunkMenue.BildBer.XMax,FunkMenue.BildBer.YMax);
  wxRect tmp(p1,p2);
  if (!finished)
  {
    if (m_moved)
    {
      wxRect tmp1(tmp);
      tmp1.x += m_lastcurpos.x-m_curpos.x;
      tmp1.y += m_lastcurpos.y-m_curpos.y;
      DrawFocusRect(DC,tmp1);
    }
    m_moved |= ((abs(m_curpos.x-hp.x) > 10) || (abs(m_curpos.y-hp.y) > 10));
    if (m_moved)
    {
      wxRect tmp1(tmp);
      tmp1.x += hp.x-m_curpos.x;
      tmp1.y += hp.y-m_curpos.y;
      DrawFocusRect(DC,tmp1);
    }
  }
  else
  {
    wxSize Abm = GetWindowDimensions();
    wxPoint delta(m_lastcurpos.x-m_curpos.x,m_lastcurpos.y-m_curpos.y);
    tmp.x += m_lastcurpos.x-m_curpos.x;
    tmp.y += m_lastcurpos.y-m_curpos.y;
    if (m_moved)
      DrawFocusRect(DC,tmp);
    if (domove)
    {
      DiagDataBaseObj tmpbase(DDiagData);
      DDiagData.LegX += delta.x*10000/Abm.x;
      if (DDiagData.LegX > 0)
        DDiagData.LegX = 0;
      DDiagData.LegY += delta.y*(ZweiDiags() ? 14140 : 7070)/Abm.y;
      if (MyMacroRecorder::Get()->IsRecording())
      {
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_MODIFYLEGEND"),GetId());
        EFXmlNode *node = MyMacroRecorder::Get()->GetAkNode();
        DDiagData.XmlWriteLegend(node,&tmpbase);
      }
      Invalidate();
      SendModified();
    }
  }
}

void TEFGraphControl::SelectGraphFromPos(wxDC &DC,wxPoint hp)
{
  if (m_hasfocus)
    DrawSizeRect(DC); // Size-Rechtecke verstecken
  int a1 = FunkMenue.HoleBefehl(hp.x,hp.y);
  if (a1 != -1)
  {
    DC.SetFont(*LegendeFont);
    SetMarkersVisible(false,DC);
    SetActiveGraph(a1,DC);
    SetMarkersVisible(true,DC);
    DDiagData.AkGraph = DDiagData.YDaten.GetAkPos();
    DiagCurPos(DC);
  }
  DC.SetUserScale(1.0,1.0);
  if (m_hasfocus)
    DrawSizeRect(DC);
}

void TEFGraphControl::OnLButtonUp(wxMouseEvent &event)
{
  wxPoint point,hp = event.GetPosition();
  ScreenToClient(hp);
  point = hp;

  if (m_capt)
  {
    ReleaseMouse();
    m_capt = false;
    if (m_movemode == GraphSelect)
    { // nothing to do, as graph was already selected in OnLButtonDown
    }
    else
    if (m_movemode == ZoomIn)
    {
      if (m_moved)
      {
        AddUndo(UndoDiagDataBase);
        DiagDataBaseObj orig(DDiagData);
        wxClientDC DC(this);
        wxRect rect(m_curpos,
                    wxSize(m_lastcurpos.x-m_curpos.x,m_lastcurpos.y-m_curpos.y));
        if (m_moved)
          DrawFocusRect(DC,rect);
        SetScreenParams(DC);
        DarstDataObj *DD = GetAkDiag().GetDarstData();
        AxisDataArr XAxis = DD->XAxis;
        AxisDataArr YAxis = DD->YAxis;

        fptr oldhndl = SetFPESignal();
        try
        {
          XAxis.Begin();
          DD->XAxis.Begin();
          XAxis->Min = GetAkDiag().XWert(hp.x);
          XAxis->Max = GetAkDiag().XWert(m_curpos.x);
          YAxis.Begin();
          DD->YAxis.Begin();
          do
          {
            if (YAxis->GetVisible())
            {
              YAxis->Min = GetAkDiag().YWert(hp.y);
              YAxis->Max = GetAkDiag().YWert(m_curpos.y);
            }
          }
          while (YAxis.NextEint() && DD->YAxis.NextEint());
        }
        catch (...)
        {
          wxMessageBox(_("Value range invalid !"),wxT("Easy Funktion"),
                       wxOK+wxICON_EXCLAMATION,wxGetApp().GetTopWindow());
        }
        ClearFPESignal(oldhndl);

        DD->SortAxis(XAxis.GetAkEint());
        YAxis.Begin();
        bool yok = true;
        do
        {
          DD->SortAxis(YAxis.GetAkEint());
          if (YAxis->GetVisible() && !DD->GetBerOK(YAxis.GetAkEint()))
            yok = false;
        }
        while (YAxis.NextEint());
        if (DD->GetBerOK(XAxis.GetAkEint()) && yok)
        {
          DD->XAxis = XAxis;
          DD->YAxis = YAxis;
          DDiagData.AutoMode &= ~(unsigned short)(DarstDataObj::AutoX+
                                        DarstDataObj::AutoY);
          UpdateWerBer();
          SendModified();
          SendDiagModified();
          if (MyMacroRecorder::Get()->IsRecording())
          {
            MyMacroRecorder::Get()->AddEvent(wxT("CMD_MODIFYDIAGRAM"),GetId());
            DiagDataBaseObj *tmp = dynamic_cast<DiagDataBaseObj *>(&DDiagData);
            tmp->XmlSpeichern(MyMacroRecorder::Get()->GetAkNode(),&orig);
          }
        }
      }
    }
    else
    if (m_movemode == Legende)
    {
      if (m_moved)
      {
        MoveLegend(hp,true);
      }
    }
    else
    if (m_movemode == Label)
    {
      if (m_moved)
      {
        MoveLabel(hp,true);
      }
    }
    else
    if (m_moved)
    {
      DrawMoveRect(m_lastcurpos,m_movemode,GetRatio());
      wxRect rect = NewRect(point,m_movemode,GetRatio());
      wxPoint pt = rect.GetPosition();
      GetParent()->ScreenToClient(&pt.x,&pt.y);
      rect.x = pt.x;
      rect.y = pt.y;
      SetSize(rect);
      SendModified();
    }
  }
  m_movemode = MoveWin;
}

void TEFGraphControl::OnSetCursor(wxSetCursorEvent &event)
{
  wxPoint point(event.GetX(),event.GetY()),hp = point;
  long akcurid = -1;
  if (m_hasfocus && !MenuCursor)
  {
    switch (PointInSizeRect(point))
    {
      case MoveWin :
      {
        int a = FunkMenue.HoleBefehl(hp.x,hp.y);
        if ((a == -2) || (m_movemode != MoveWin)) // In Legende
          akcurid = wxCURSOR_SIZING;
        else
        if ((a >= 0) && LegendeCursor) // In MenueItem in Legende
          m_csr = *LegendeCursor;
        else
        if (FindLabel(hp.x,hp.y)) // In Label
          akcurid = wxCURSOR_SIZING;
        else
        if ((Diag1.PunktInLabel(hp.x,hp.y) ||
            ZweiDiags() &&
            Diag2.PunktInLabel(hp.x,hp.y)) && LegendeCursor)
          m_csr = *LegendeCursor;
        else
        if (Diag1.PunktInObject(hp.x,hp.y) ||
            ZweiDiags() &&
            Diag2.PunktInObject(hp.x,hp.y))
        {
          if (wxGetApp().GetZoomIn())
            akcurid = wxCURSOR_MAGNIFIER;
          else
          if (DiagrammCursor)
            m_csr = *DiagrammCursor;
        }
        else
          akcurid = wxCURSOR_SIZING;
        break;
      }
      case Top :
      case Bottom :
        akcurid = wxCURSOR_SIZENS;
        break;
      case Left :
      case Right :
        akcurid = wxCURSOR_SIZEWE;
        break;
      case TopRight :
      case BottomLeft :
        akcurid = wxCURSOR_SIZENESW;
        break;
      case BottomRight :
      case TopLeft :
        akcurid = wxCURSOR_SIZENWSE;
        break;
      default : m_csr = *wxSTANDARD_CURSOR;
    }
  }
  else
  if (!m_hasfocus && !MenuCursor)
    akcurid = wxCURSOR_SIZING;
  else
    m_csr = *wxSTANDARD_CURSOR;
  if ((akcurid != m_curId) || (akcurid == -1))
  {
    m_curId = akcurid;
    if (m_curId != -1)
      m_csr = wxCursor(m_curId);
#ifdef __WXGTK__
    SetCursor(m_csr);
#endif
  }
#ifdef __WXMSW__
  event.SetCursor(m_csr);
#endif
}

void TEFGraphControl::OnKeyDown(wxKeyEvent &event)
{
  if (event.AltDown())
  {
    event.Skip();
    return;
  }
  wxClientDC DC(this);
  DrawSizeRect(DC);
  SetScreenParams(DC);
  DC.SetFont(*LegendeFont);
  switch (event.GetKeyCode())
  {
    case WXK_ESCAPE :
    case WXK_RETURN :
      if (m_movemode == Legende)
      {
        MoveLegend(m_lastcurpos,true,event.GetKeyCode() == WXK_RETURN);
        ReleaseMouse();
        m_capt = false;
      }
      else
      if (m_movemode == Label)
      {
        MoveLabel(m_lastcurpos,true,event.GetKeyCode() == WXK_RETURN);
        ReleaseMouse();
        m_capt = false;
      }
      else
        event.Skip();
      m_movemode = MoveWin;
      Invalidate();
      break;
    case WXK_NUMPAD6 :
      GetAkDiag().Move(EFDiagObj::SchnellVorWaerts,0,DC);
      break;
    case WXK_NUMPAD4 :
      GetAkDiag().Move(EFDiagObj::SchnellRueckWaerts,0,DC);
      break;
    case WXK_RIGHT :
      if (!event.ControlDown())
      {
        wxPoint tmp = m_lastcurpos;
        tmp.x += 5;
        if (m_movemode == Legende)
          MoveLegend(tmp,false);
        else
        if (AkLabel && (m_movemode == Label))
          MoveLabel(tmp,false);
        else
          GetAkDiag().Move(EFDiagObj::VorWaerts,0,DC);
        m_lastcurpos = tmp;
      }
      else
        event.Skip();
      break;
    case WXK_LEFT :
      if (!event.ControlDown())
      {
        wxPoint tmp = m_lastcurpos;
        tmp.x -= 5;
        if (m_movemode == Legende)
          MoveLegend(tmp,false);
        else
        if (AkLabel && (m_movemode == Label))
          MoveLabel(tmp,false);
        else
          GetAkDiag().Move(EFDiagObj::RueckWaerts,0,DC);
        m_lastcurpos = tmp;
      }
      else
        event.Skip();
      break;
    case WXK_UP :
      if (!event.ControlDown())
      {
        wxPoint tmp = m_lastcurpos;
        tmp.y -= 5;
        if (m_movemode == Legende)
          MoveLegend(tmp,false);
        else
        if (AkLabel && (m_movemode == Label))
          MoveLabel(tmp,false);
        else
        if (event.ShiftDown())
        {
          unsigned char NewDiag = 0;
          MarkDiag(wxPoint(0,0),DC,true,&NewDiag);
        }
        else
        {
          SetMarkersVisible(false,DC);
          SetActiveGraph(char(DDiagData.YDaten.GetAkPos()-1),DC);
          SetMarkersVisible(true,DC);
          DDiagData.AkGraph = DDiagData.YDaten.GetAkPos();
        }
        m_lastcurpos = tmp;
      }
      else
        event.Skip();
     break;
    case WXK_DOWN :
      if (!event.ControlDown())
      {
        wxPoint tmp = m_lastcurpos;
        tmp.y += 5;
        if (m_movemode == Legende)
          MoveLegend(tmp,false);
        else
        if (AkLabel && (m_movemode == Label))
          MoveLabel(tmp,false);
        else
        if (event.ShiftDown())
        {
          unsigned char NewDiag = (!ZweiDiags() ? 0 : 1);
          MarkDiag(wxPoint(0,0),DC,true,&NewDiag);
        }
        else
        {
          SetMarkersVisible(false,DC);
          SetActiveGraph(char(DDiagData.YDaten.GetAkPos()+1),DC);
          SetMarkersVisible(true,DC);
          DDiagData.AkGraph = DDiagData.YDaten.GetAkPos();
        }
        m_lastcurpos = tmp;
      }
      else
        event.Skip();
      break;
    case WXK_DELETE :
      if (event.ControlDown() || !DDiagData.YDaten.YDatenVorhanden())
      {
        wxCommandEvent cevent;
        OnDelDiag(cevent);
      }
      else
      {  
        AddUndo(UndoDDiagData);
        GraphenLoeschen(false);
        if (MyMacroRecorder::Get()->IsRecording())
          MyMacroRecorder::Get()->AddEvent(wxT("CMD_DELETEGRAPH"),GetId());
      }  
      break;
    default:
      event.Skip();
  }
  DiagCurPos(DC);
  DC.SetUserScale(1.0,1.0);
  DrawSizeRect(DC);
}

void TEFGraphControl::OnSize(wxSizeEvent &event)
{
  wxClientDC dc(this);
  SetScreenParams(dc);
  UpdateDiag(dc);
  Show(true);
  event.Skip();
}

void TEFGraphControl::OnPaint(wxPaintEvent &)
{
  Diag1.Text(true);
  wxPaintDC dc(this);

  wxSize size = GetClientSize();

  Paint(dc,false,size);
}

// Alle Bezeichnerfelder des aktuellen Diagramms loeschen
void TEFGraphControl::OnDelAllLabels(wxCommandEvent &)
{
  AddUndo(UndoDiagDataBase);
  DDiagData.Labels.Begin();
  bool next;
  if (DDiagData.Labels.GetMaxEintrag() >= 0)
    do
    {
      if (((DDiagData.Labels->Flags & LabelObj::Diag2) > 0) ==
          (AkDiag > 0))
      {
        int AkEint = DDiagData.Labels.AkEintNummer();
        DDiagData.Labels.LoeschAkEint();
        next = DDiagData.Labels.SetAkEint(AkEint);
      }
      else
        next = DDiagData.Labels.NextEint();
    }
    while (next);
  SendModified();
  Refresh();
  AkLabel = 0;
}

void TEFGraphControl::OnDelLabel(wxCommandEvent &)
{
  if (AkLabel)
  {
    AddUndo(UndoDiagDataBase);
    DDiagData.Labels.Begin();
    if (DDiagData.Labels.GetMaxEintrag() >= 0)
      do
        if (&DDiagData.Labels.GetAkEint() == AkLabel)
        {
          DDiagData.Labels.LoeschAkEint();
          SendModified();
          Refresh();
          AkLabel = 0;
          return;
        }
      while (DDiagData.Labels.NextEint());
  }
}

bool TEFGraphControl::SetActiveLabel(size_t index)
{
  if (!index)
    return false;
  if (MyMacroRecorder::Get()->IsRecording())
  {
    DDiagData.Labels.SetAkEint(index-1);
    if (AkLabel != &DDiagData.Labels.GetAkEint())
    {
      MyMacroRecorder::Get()->AddEvent(wxT("CMD_SELECTLABEL"),GetId());
      MGPStr tmp;
      tmp.AddInt(index);
      MyMacroRecorder::Get()->GetAkNode()->AddProperty(wxT("index"),tmp.c_str());
    }
  }
  DDiagData.Labels.SetAkEint(index-1);
  AkLabel = &DDiagData.Labels.GetAkEint();
  return true;
}

void TEFGraphControl::OnLabelOptions(wxCommandEvent &)
{
  if (AkLabel)
    LabelOptions(*AkLabel);
}

void TEFGraphControl::OnLabelSelect(wxCommandEvent &)
{
  wxString *Options = new wxString[DDiagData.Labels.GetMaxEintrag()+1];

  DDiagData.Labels.Begin();
  if (DDiagData.Labels.GetMaxEintrag() >= 0)
    do
    {
      Options[DDiagData.Labels.AkEintNummer()] =
        DDiagData.Labels.GetAkEint().Label.c_str();
    }
    while (DDiagData.Labels.NextEint());
  wxSingleChoiceDialog dlg(wxGetApp().GetTopWindow(),_("Chose label:"),
                           wxT("Easy Funktion"),
                           DDiagData.Labels.GetMaxEintrag(),Options);
  if (dlg.ShowModal() != wxID_CANCEL)
  {
    DDiagData.Labels.SetAkEint(dlg.GetSelection());
    SetActiveLabel(DDiagData.Labels.AkEintNummer()+1);
    wxMenu *menu = wxXmlResource::Get()->LoadMenu(wxT("menuLabel"));
    MenuCursor = true;
    wxPoint hp = wxGetMousePosition();
    hp = ScreenToClient(hp);
    PopupMenu(menu,hp);
    MenuCursor = false;
    delete menu;
  }
  delete[] Options;
}

bool TEFGraphControl::LabelOptions(LabelObj &Label,LabelArrObj *org)
{
  bool ret = false;
  AddUndo(UndoDiagDataBase);
  LabelArrObj *orglabels = org;
  if (!org)
    orglabels = new LabelArrObj(DDiagData.Labels);
  TEFLabelDlg dlg(Label);
  dlg.Create(wxGetApp().GetTopWindow(),dlgLabel,_("Label options"));
  if (dlg.ShowModal() == wxID_OK)
  {
    wxClientDC DC(this);
    SetScreenParams(DC);
    LabelSizeNDraw(DC,Label,0,0,false);
    if (MyMacroRecorder::Get()->IsRecording())
    {
      if (org)
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_ADDLABEL"),GetId());
      else
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_MODIFYLABEL"),GetId());
      EFXmlNode *node = MyMacroRecorder::Get()->GetAkNode();
      DDiagData.Labels.XmlWrite(node,orglabels);
    }
    SendModified();
    Refresh();
    ret = true;
  }
  else
    DelUndo();
  if (!org)
    delete orglabels;
  return ret;
}

void TEFGraphControl::OnAddLabel(wxCommandEvent &)
{
  AddUndo(UndoDiagDataBase);
  DDiagData.Labels.NeuAkEint();
  LabelObj &Label = DDiagData.Labels.GetAkEint();
  DarstDataObj *DD = GetAkDiag().GetDarstData();
  int Nummer = GetAkDiag().GetNummer();
  double MarkX,MarkY;
  if (DDiagData.YDaten.YDatenVorhanden() &&
      (GetAkDiag().GetNummer() < DDiagData.YDaten.AkGraph().GetPunkte()))
  {
    MarkX = DDiagData.YDaten.AkGraph().GetX(Nummer);
    double HiX = MarkX;
    MarkY = DD->YUmrechnen(DDiagData.YDaten.AkGraph().GetY(Nummer),HiX);
  }
  else
  {
    MarkX = 0;//GetAkDiag().XAxis->Min;
    MarkY = 0;//GetAkDiag().YAxis->Min;
  }
  LabelArrObj orglabels(DDiagData.Labels);
  int Flags = LabelObj::ShowLabel |
              LabelObj::ShowX | LabelObj::ShowY |
              LabelObj::Frame | LabelObj::Arrow |
              (AkDiag == 1 ? LabelObj::Diag2 : 0);
  DDiagData.Labels->Init(wxT(""),Flags,MarkX,MarkY);
  if (!LabelOptions(Label,&orglabels))
    DDiagData.Labels.LoeschAkEint();
  else
  {
    wxClientDC DC(this);
    SetScreenParams(DC);
    LabelSizeNDraw(DC,Label,0,0,false);
    LabelIntoSight(Label);
  }
  SendModified();
  Refresh();
}

void TEFGraphControl::OnLabelsIntoSight(wxCommandEvent &)
{
  AddUndo(UndoDiagDataBase);
  DDiagData.Labels.Begin();
  bool m_moved = false;
  if (DDiagData.Labels.GetMaxEintrag() >= 0)
    do
      m_moved |= LabelIntoSight(DDiagData.Labels.GetAkEint());
    while (DDiagData.Labels.NextEint());
  if (m_moved)
  {
    SendModified();
    Refresh();
  }
}

// Label in sichtbaren Bereich schieben
bool TEFGraphControl::LabelIntoSight(LabelObj &Label)
{
  wxPoint pos = GetLabelPos(Label);
  wxSize size = GetLabelSize(Label);
  const GraphBaseObj::BildType &BildBer = GetLabelDiag(Label)->GetBildBer();
  int x = 0,y = 0;
  if (pos.x < BildBer.XMin)
    x = BildBer.XMin-pos.x;
  if (pos.y < BildBer.YMin)
    y = BildBer.YMin-pos.y;
  if (pos.x+size.x > BildBer.XMax)
    x = BildBer.XMax-pos.x-size.x;
  if (pos.y+size.y > BildBer.YMax)
    y = BildBer.YMax-pos.y-size.y;
  if (x || y)
  {
    MoveLabel(Label,x,y);
    return true;
  }
  return false;
}

void TEFGraphControl::OnDelDiag(wxCommandEvent &)
{
  if (!MenuCursor)
  {
    wxCommandEvent cevent(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_DELWINDOW"));
    cevent.SetEventObject(this);
    GetParent()->GetEventHandler()->AddPendingEvent(cevent);
  }
  else
    DelDiag = true;
}

void TEFGraphControl::OnCopyGraph(wxCommandEvent &)
{
#ifdef __WXGTK__
  if (wxTheClipboard->IsOpened())
    return;
#endif
  if (wxTheClipboard->Open())
  {
    wxTheClipboard->Clear();
    wxDataObjectComposite *cmp = new wxDataObjectComposite;
    CopyGraph(*cmp);
    wxTheClipboard->SetData(cmp);
    wxTheClipboard->Close();
  }
}

void TEFGraphControl::OnEditCopy(wxCommandEvent &event)
{
  wxString Options[] =
  {
    _("Blank diagram"),
    _("Current graph"),
    _("All graphs"),
    _("Diagram with graphs"),
  };

  int item = 0;
  if (DDiagData.YDaten.YDatenVorhanden())
  {
    wxSingleChoiceDialog dlg(wxGetApp().GetTopWindow(),_("Chose item to copy:"),
                             wxT("Easy Funktion"),4,Options);
    if (dlg.ShowModal() == wxID_CANCEL)
      return;
    item = dlg.GetSelection();
  }
  switch (item)
  {
    case 0: // Diagramm ins ClipBoard
      if (wxTheClipboard->Open())
      {
        wxTheClipboard->Clear();
        wxDataObjectComposite *cmp = new wxDataObjectComposite;
        CopyDiag(*cmp);
        wxTheClipboard->SetData(cmp);
        wxTheClipboard->Close();
      }
      break;
    case 1: // Graph ins ClipBoard
      OnCopyGraph(event);
      break;
    case 2: // Alle Graphen ins ClipBoard
      if (wxTheClipboard->Open())
      {
        wxTheClipboard->Clear();
        int size = 0;
        void *tmp = DDiagData.YDaten.CopyAllGraphs(Parser,size);
        wxCustomDataObject *doo;
        doo = new wxCustomDataObject(wxGetApp().graphformat);
        doo->SetData(size,tmp);
        wxTheClipboard->SetData(doo);
        free(tmp);
        wxTheClipboard->Close();
      }
      break;
    case 3: // Alle Graphen und Diagramm ins ClipBoard
      if (wxTheClipboard->Open())
      {
        wxTheClipboard->Clear();
        wxDataObjectComposite *cmp = new wxDataObjectComposite;
#ifdef __WXMSW__
        wxEnhMetaFileDC dc;
        wxSize Abm = GetWindowDimensions(true);
        wxSize size = GetClientSize();
        SetScreenParams(dc,size.x,size.y);
        UpdateDiag(dc,true);
        const wxBrush &OldBr = dc.GetBrush();
        wxBrush brs(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW),
                    wxSOLID);
        dc.SetBrush(brs);
        wxPen pen(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW),1,
                  wxSOLID);
        dc.SetPen(pen);
        dc.DrawRectangle(0,0,Abm.x,Abm.y); // Hintergrund in Weiss
        dc.SetPen(wxNullPen);
        dc.SetBrush(OldBr);
        BildschirmAusgabe(dc,false,true);
        wxEnhMetaFile *mf = dc.Close();
        if (mf)
        {
          wxEnhMetaFileSimpleDataObject *data;
          data = new wxEnhMetaFileSimpleDataObject(*mf);
          cmp->Add(data,true);
          delete mf;
        }
#endif
        int dsize = 0;
        void *tmp = DDiagData.YDaten.CopyAllGraphs(Parser,dsize);
        wxCustomDataObject *doo;
        doo = new wxCustomDataObject(wxGetApp().graphformat);
        CopyDiag(*cmp);
        doo->SetData(dsize,tmp);
        cmp->Add(doo);
        wxTheClipboard->SetData(cmp);
        free(tmp);
        wxTheClipboard->Close();
      }
      Invalidate();
      break;
  }
}

void TEFGraphControl::OnDiag2(wxCommandEvent &event)
{
  DarstDataObj *DD;

  if (event.IsChecked())
  {
    SetAkDiag(1);
    if (!MyMacroRecorder::Get()->IsPlaying())
      OnDarstellung(event);
  }
  else
  {
    AddUndo(UndoDiagDataBase);
    SetAkDiag(0);
    DD = GetDiag(1)->GetDarstData();
    DD->Init(DarstDataObj::einzel,1,DD->Mode,
             DD->Titel.c_str(),DD->NumDigits);
    UpdateWerBer();
    SendModified();
  }
}

bool TEFGraphControl::PasteGraph(wxCustomDataObject &cdo,bool dragndrop)
{
  void *tmp = cdo.GetData();
  size_t usedsize = 0;
  int RetStatus;
  do // hack fuer ungenaues GetSize()
  {
    GraphDataObj Ydata;
    usedsize += Ydata.ConvertFromMemBlock(&((char *)tmp)[usedsize]);
    MGPStr tmp(Ydata.FuString);
    RetStatus = CheckGraphName(tmp,0,dragndrop);
    _tcscpy(Ydata.FuString,tmp.c_str());
    if (RetStatus == wxID_OK)
      DDiagData.YDaten.AppendYFeld(&Ydata,true);
  }
  while ((cdo.GetSize() > sizeof(GraphDataObj)+usedsize) && (RetStatus == wxID_OK));
  UpdateWerBer();
  SendModified();
  return (RetStatus == wxID_OK);
}

void TEFGraphControl::OnEditPaste(wxCommandEvent &)
{
#ifdef __WXGTK__
  if (wxTheClipboard->IsOpened())
    return;
#endif
  if (wxTheClipboard->Open())
  {
    AddUndo(UndoDDiagData);
    bool dirty = false;
    if (wxTheClipboard->IsSupported(wxGetApp().diagformat))
    { // wenn Diagramm im Clipboard, Format kopieren
      wxCustomDataObject cdo(wxGetApp().diagformat);
      wxTheClipboard->GetData(cdo);
      DiagDataObj *Pt = (DiagDataObj *)cdo.GetData();
      DDiagData.Copy(*Pt,false);
      SetSize(wxRect(Pt->WinXPos+20,Pt->WinYPos+20,Pt->WinWidth,Pt->WinHeight)),
      dirty = true;
    }
    if (wxTheClipboard->IsSupported(wxGetApp().graphformat))
    { // Wenn Graph im binaeren Format im Clipboard
      wxCustomDataObject cdo(wxGetApp().graphformat);
      wxTheClipboard->GetData(cdo);
      PasteGraph(cdo);
      dirty = true;
    }
    else
    if (wxTheClipboard->IsSupported(wxDF_TEXT))
    { // wenn Text im Clipboard, mit LoadGraph laden lassen
      wxTextDataObject data;
      wxTheClipboard->GetData( data );
      wxBeginBusyCursor();
      bool ret = DDiagData.YDaten.LoadGraph(0,data.GetText().c_str(),
                                            true,true,false,true);
      wxEndBusyCursor();
      if (!ret)
      {
        wxMessageBox(_("Invalid entry !"),wxT("Easy Funktion"),
                     wxOK+wxICON_EXCLAMATION,wxGetApp().GetTopWindow());
      }
      else
        dirty = true;
    }
    if (dirty)
    {
      UpdateWerBer();
      SendModified();
    }
    wxTheClipboard->Close();
  }
}

void TEFGraphControl::CeEditPaste(wxUpdateUIEvent& ce)
{
  ce.Enable(false);
#ifdef __WXGTK__
  if (wxTheClipboard->IsOpened())
    return;
#endif
  ce.Enable(wxTheClipboard->Open() &&
            (wxTheClipboard->IsSupported(wxDF_TEXT) ||
             wxTheClipboard->IsSupported(wxGetApp().graphformat) ||
             wxTheClipboard->IsSupported(wxGetApp().diagformat)));
  wxTheClipboard->Close();
}

void TEFGraphControl::OnGraphOptionen(wxCommandEvent &)
{
  GraphOptionen(false);
}

void TEFGraphControl::OnGraphTable(wxCommandEvent &)
{
  GraphOptionen(false,true);
}

void TEFGraphControl::OnGraphWerte(wxCommandEvent &)
{
  GraphOptionen(true);
}

void TEFGraphControl::OnDarstellung(wxCommandEvent &)
{
  DarstellungsDialog(0,0);
}

void TEFGraphControl::DarstellungsDialog(int DlgPage,int YAxisNr)
{
  int RetStatus;
  bool OK;
  AxisDataArr OldXAxis,OldYAxis;
  DarstDataObj *DD;
  TEFDarstellungDlg *Dlg;
  DarstellungsData DDD;
  wxString Zeile;

  DiagDataBaseObj orig(DDiagData);
  DD = GetAkDiag().GetDarstData();
  DDD.DiagText.SetStr(DD->Titel.c_str());
  DDD.Mode = DD->Mode;
  DDD.XAxis = DD->XAxis;
  DDD.XAxis.Begin();
  DDD.YAxis = DD->YAxis;
  DDD.YAxis.Begin();
  DDD.SameXAndYScaling = DD->m_SameXAndYScaling;

  DDD.Achse = DDiagData.Achse;
  DDD.Titel = DDiagData.Titel;
  DDD.AchsLabels = DDiagData.AchsLabels;
  DDD.EData = *((EinstData *)&DDiagData);
  DDD.NumDigits = DD->NumDigits;
  DDD.PostProc = DDiagData.PostProc;
  DDD.PostTerm = DDiagData.PostProcTerm.c_str();
  DDD.DiagPage = DlgPage;
  DDD.YAxisNr = YAxisNr;
  DDD.Colour = DDiagData.m_Colour;
  if (DD->Conversion == DarstDataObj::einzel)
    DDD.Conversion = DarstDataObj::realY;
  else
    DDD.Conversion = DD->Conversion;
  OldXAxis = DDD.XAxis;
  OldYAxis = DDD.YAxis;
  OK = true;
  do
  {
    Dlg = new TEFDarstellungDlg(DDD);
    RetStatus = wxID_CANCEL;
    if (Dlg)
    {
      Zeile = _("Diagram properties");
      if (ZweiDiags())
      {
        if (AkDiag == 0)
          Zeile += wxT(" #1");
        else
          Zeile += wxT(" #2");
      }
      Dlg->Create(wxGetApp().GetTopWindow(),dlgDarstellung,Zeile);
      RetStatus = Dlg->ShowModal();
      Dlg->Destroy();
    }
    if (RetStatus == wxID_OK)
    {
      OK = true;

      DDD.XAxis.Begin();
      OldXAxis.Begin();
      do
        DD->SortAxis(DDD.XAxis.GetAkEint());
      while (DDD.XAxis.NextEint());
      if ((DDD.XAxis->Min != OldXAxis->Min) ||
          (DDD.XAxis->Max != OldXAxis->Max))
      { // automatische Bereichs-Berechnung aus wenn manuell geaendert
        DDiagData.AutoMode &= ~(unsigned short)(DarstDataObj::AutoX);
      }

      DDD.YAxis.Begin();
      OldYAxis.Begin();
      do
      {
        DD->SortAxis(DDD.YAxis.GetAkEint());
        if ((DDD.YAxis->Min != OldYAxis->Min) ||
            (DDD.YAxis->Max != OldYAxis->Max))
        { // automatische Bereichs-Berechnung aus wenn manuell geaendert
          DDiagData.AutoMode &= ~(unsigned short)(DarstDataObj::AutoY);
        }
      }
      while (DDD.YAxis.NextEint() && OldYAxis.NextEint());
    }

    if ((DDD.EData.Ma.Anf < DDD.EData.Ma.Ende) && (DDD.EData.Ma.Step < 0) ||
       (DDD.EData.Ma.Anf > DDD.EData.Ma.Ende) && (DDD.EData.Ma.Step > 0))
    {
      double Hi = DDD.EData.Ma.Anf;
      DDD.EData.Ma.Anf = DDD.EData.Ma.Ende;
      DDD.EData.Ma.Ende = Hi;
    }
    if ((abs(DDD.EData.Ma.Ende-DDD.EData.Ma.Anf) <
         MaTyp::FourIt*DDD.EData.Ma.Step) &&
        (DDD.EData.Ma.Mode == MaTyp::ModeFourier) ||
       (abs(DDD.EData.Ma.Ende-DDD.EData.Ma.Anf) <
        MaTyp::SumIt*DDD.EData.Ma.Step) &&
       (DDD.EData.Ma.Mode == MaTyp::ModeSum) ||
       (abs(DDD.EData.Ma.Ende-DDD.EData.Ma.Anf) <
        MaTyp::ParIt*DDD.EData.Ma.Step) &&
       (DDD.EData.Ma.Mode == MaTyp::ModePar) ||
       (DDD.EData.Ma.Mode == MaTyp::ModeNo) ||
       (DDD.EData.Ma.Mode == MaTyp::ModeVar) ||
       (DDD.EData.Ma.Mode == MaTyp::ModeParList))
    {
    }
    else
    {
      wxMessageBox(_("Too many steps for 'Hi' !"),wxT("Easy Funktion"),
                   wxOK+wxICON_EXCLAMATION,wxGetApp().GetTopWindow());
      OK = false;
    }

    if ((DDD.EData.Punkte <= 1) || (DDD.EData.Punkte > GraphDataObj::MaxPunkte))
    {
      wxString Zeile;
      Zeile.Printf(_("Number of points must be between 2 and %i !"),
              GraphDataObj::MaxPunkte);
      wxMessageBox(Zeile,wxT("Easy Funktion"),wxOK+wxICON_EXCLAMATION,
                   wxGetApp().GetTopWindow());
      OK = false;
    }

    if (DDD.EData.CXMin > DDD.EData.CXMax)
    {
      double Hi = DDD.EData.CXMin;
      DDD.EData.CXMin = DDD.EData.CXMax;
      DDD.EData.CXMax = Hi;
    }
    if ((DDD.EData.CXMin == DDD.EData.CXMax))
    {
      wxMessageBox(_("Invalid X range !"),wxT("Easy Funktion"),
                   wxOK+wxICON_EXCLAMATION,wxGetApp().GetTopWindow());
      OK = false;
    }

    DD->NumDigits = DDD.NumDigits;
    if (OK)
      *((EinstData *)&DDiagData) = DDD.EData;

    DDD.YAxis.Begin();
    do
    {
      if (DDD.Conversion == DarstDataObj::logY)
        DDD.YAxis->SetLog(true);
      if ((DDD.YAxis->Min < 0) && DDD.YAxis->GetLog() &&
          (DDiagData.AutoMode & DarstDataObj::AutoY))
      { // ungueltiger Y-Wertebereich bei autom. Berechnung
        DDD.YAxis->Max = 10; // Default-Fallback-Werte eintragen
        DDD.YAxis->Min = 1;
      }
      wxString tmp;
      tmp.Printf(_("Y axis %i"),DDD.YAxis.AkEintNummer());
      if (((DDD.YAxis->Min < 0) && DDD.YAxis->GetLog()))
      { // ungueltiger Y-Wertebereich und nicht automatische Berechnung
        wxMessageBox(_("In logarithmic mode only positive values !"),
                     tmp,wxOK+wxICON_EXCLAMATION,
                     wxGetApp().GetTopWindow());
        OK = false;
      }
      else
      if (!DD->GetBerOK(DDD.YAxis.GetAkEint()))
      {
        wxMessageBox(_("Value range invalid !"),tmp,
                     wxOK+wxICON_EXCLAMATION,wxGetApp().GetTopWindow());
        OK = false;
      }
    }
    while (DDD.YAxis.NextEint());

    DDD.XAxis.Begin();
    if ((DDD.XAxis->Min < 0) && DDD.XAxis->GetLog() &&
        (DDiagData.AutoMode & DarstDataObj::AutoX))
    { // ungueltiger X-Wertebereich bei autom. Berechnung
      DDD.XAxis->Max = 10; // Default-Fallback-Werte eintragen
      DDD.XAxis->Min = 1;
    }
    if (((DDD.XAxis->Min < 0) && DDD.XAxis->GetLog()))
    { // ungueltiger X-Wertebereich und nicht automatische Berechnung
      wxMessageBox(_("In logarithmic mode only positive values !"),
                   _("X axis"),wxOK+wxICON_EXCLAMATION,
                   wxGetApp().GetTopWindow());
      OK = false;
    }
    else
    if (!DD->GetBerOK(DDD.XAxis.GetAkEint()))
    {
      wxMessageBox(_("Value range invalid !"),_("X axis"),
                   wxOK+wxICON_EXCLAMATION,wxGetApp().GetTopWindow());
      OK = false;
    }
    else
    if (OK && (RetStatus == wxID_OK))
    {
      AddUndo(UndoDiagDataBase+UndoPostProcFunction);
      DDiagData.Achse = DDD.Achse;
      DDiagData.Titel = DDD.Titel;
      DDiagData.AchsLabels = DDD.AchsLabels;
      DDiagData.m_Colour = DDD.Colour;
      GetAkDiag().SetMode(DD->Mode);
      Diag1.SetAchseFont(DDD.Achse);
      Diag2.SetAchseFont(DDD.Achse);
      Diag1.SetTitelFont(DDD.Titel);
      Diag2.SetTitelFont(DDD.Titel);
      GetAkDiag().SetNumDigits(DDD.NumDigits);

      DD->XAxis = DDD.XAxis;
      DD->XAxis.Begin();
      DD->YAxis = DDD.YAxis;
      DD->YAxis.Begin();
      DD->Init(DDD.Conversion,
               DD->DiagNr,DDD.Mode,
               DDD.DiagText.c_str(),DDD.NumDigits);
      DD->m_SameXAndYScaling = DDD.SameXAndYScaling;
      DDiagData.SetPostProc(DDD.PostProc);
      DDiagData.PostProcTerm.SetStr(DDD.PostTerm.c_str());
      PostProcFunktion->Term.SetStr(DDD.PostTerm.c_str());
      if (MyMacroRecorder::Get()->IsRecording())
      {
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_MODIFYDIAGRAM"),GetId());
        DiagDataBaseObj *tmp = dynamic_cast<DiagDataBaseObj *>(&DDiagData);
        tmp->XmlSpeichern(MyMacroRecorder::Get()->GetAkNode(),&orig);
      }
      UpdateWerBer();
    }
  }
  while (!OK);
  if (RetStatus != wxID_CANCEL)
  {
    SendModified();
    SendDiagModified();
  }
}

void TEFGraphControl::OnLoeschen(wxCommandEvent &)
{
  AddUndo(UndoDDiagData);
  GraphenLoeschen(false);
}

void TEFGraphControl::OnAlleLoeschen(wxCommandEvent &)
{
  AddUndo(UndoDDiagData);
  GraphenLoeschen(true);
}

void TEFGraphControl::OnMarke(wxCommandEvent &event)
{
  wxClientDC DC(this);
  SetScreenParams(DC);
  GetAkDiag().Markieren(DC,event.IsChecked());
  DiagCurPos(DC);
}

void TEFGraphControl::OnRechts(wxCommandEvent &)
{
  wxClientDC DC(this);
  SetScreenParams(DC);
  GetAkDiag().Move(EFDiagObj::Rechts,0,DC);
  DiagCurPos(DC);
}

void TEFGraphControl::OnLinks(wxCommandEvent &)
{
  wxClientDC DC(this);
  SetScreenParams(DC);
  GetAkDiag().Move(EFDiagObj::Links,0,DC);
  DiagCurPos(DC);
}

void TEFGraphControl::OnMax(wxCommandEvent &)
{
  wxClientDC DC(this);
  SetScreenParams(DC);
  GetAkDiag().Move(EFDiagObj::Max,0,DC);
  DiagCurPos(DC);
}

void TEFGraphControl::OnMin(wxCommandEvent &)
{
  wxClientDC DC(this);
  SetScreenParams(DC);
  GetAkDiag().Move(EFDiagObj::Min,0,DC);
  DiagCurPos(DC);
}

void TEFGraphControl::OnSpitze(wxCommandEvent &)
{
  wxClientDC DC(this);
  SetScreenParams(DC);
  GetAkDiag().Move(EFDiagObj::Peak,0,DC);
  DiagCurPos(DC);
}

void TEFGraphControl::OnAutoX(wxCommandEvent &event)
{
  AddUndo(UndoDiagDataBase);
  if (event.IsChecked())
  {
    DDiagData.AutoMode |= DarstDataObj::AutoX;
    UpdateWerBer();
  }
  else
    DDiagData.AutoMode &= ~DarstDataObj::AutoX;
  SendModified();
}

void TEFGraphControl::OnAutoY(wxCommandEvent &event)
{
  AddUndo(UndoDiagDataBase);
  if (event.IsChecked())
  {
    DDiagData.AutoMode |= DarstDataObj::AutoY;
    UpdateWerBer();
  }
  else
    DDiagData.AutoMode &= ~DarstDataObj::AutoY;
  SendModified();
}

void TEFGraphControl::OnIncrease(wxCommandEvent &)
{
  AddUndo(UndoDiagDataBase);
  DarstDataObj *DD = &DDiagData.DarstData[AkDiag];
  DDiagData.AutoMode &= ~DarstDataObj::AutoX;
  DDiagData.AutoMode &= ~DarstDataObj::AutoY;

  DD->YAxis.Begin();
  do
    if (DD->YAxis->GetVisible())
      DDiagData.DarstData[AkDiag].ScaleBer(DD->YAxis.GetAkEint(),1.0+1.8/12.6);
  while (DD->YAxis.NextEint());

  DD->XAxis.Begin();
  DDiagData.DarstData[AkDiag].ScaleBer(DD->XAxis.GetAkEint(),1.0+1.8/12.6);

  UpdateWerBer();
  SendModified();
}

void TEFGraphControl::OnDecrease(wxCommandEvent &)
{
  AddUndo(UndoDiagDataBase);
  DarstDataObj *DD = &DDiagData.DarstData[AkDiag];
  DDiagData.AutoMode &= ~DarstDataObj::AutoX;
  DDiagData.AutoMode &= ~DarstDataObj::AutoY;

  DD->YAxis.Begin();
  do
    if (DD->YAxis->GetVisible())
      DDiagData.DarstData[AkDiag].ScaleBer(DD->YAxis.GetAkEint(),0.8);
  while (DD->YAxis.NextEint());

  DD->XAxis.Begin();
  DDiagData.DarstData[AkDiag].ScaleBer(DD->XAxis.GetAkEint(),0.8);

  UpdateWerBer();
  SendModified();
}

void TEFGraphControl::OnPostProc(wxCommandEvent &event)
{
  AddUndo(UndoDiagDataBase);
  DDiagData.SetPostProc(event.IsChecked());
  UpdateWerBer();
  SendModified();
}

void TEFGraphControl::OnFourierDialog(wxCommandEvent &)
{
  int State;
  AxisDataArr AkXAxis,AkYAxis;
  FourierData FD;
  TEFFourierDlg *F;
  DarstDataObj *DD = &DDiagData.DarstData[AkDiag];

  AkXAxis = DD->XAxis;
  AkYAxis = DD->YAxis;
  DiagDataBaseObj orig(DDiagData);
  FD.State = 0;
  FD.Ma = DDiagData.Ma;
  F = new TEFFourierDlg(FD);
  State = wxID_CANCEL;
  if (F)
  {
    F->Create(wxGetApp().GetTopWindow(),dlgFourier,_("Fourier options"));
    State = F->ShowModal();
    F->Destroy();
  }
  if (State == wxID_OK)
  {
    AddUndo(UndoDiagDataBase+UndoPostProcFunction);
    if (!(FD.State & TEFFourierDlg::FourOK))
    {
      wxMessageBox(_("Invalid values entered, operation cancelled."),
                   wxT("Easy Funktion"),wxOK+wxICON_EXCLAMATION,
                   wxGetApp().GetTopWindow());
    }
    else
    {
      FD.Ma.Mode = MaTyp::ModeFourier;
      FD.State &= ~int(TEFFourierDlg::FourOK);
      if ((FD.State == 0) || (FD.State == 1))
      {
        AkXAxis.Begin();
        AkXAxis->Min = -0.75/FD.Ma.Anf;
        AkXAxis->Max = 0.75/FD.Ma.Anf;
        AkXAxis->Label.SetStr(wxT("[s]"));
        AkXAxis->SetLog(false);

        AkYAxis.Begin();
        AkYAxis->Min = -1.3;
        AkYAxis->Max = 1.3;
        AkYAxis->Label.SetStr(wxT(""));
        AkYAxis->SetLog(false);
        DD = &(DDiagData.DarstData)[0];
        if (FD.State == 0)
        {
          wxString str(_("Fourier summing-Values -1 to 1"));
          DD->Init(DarstDataObj::realY,0,
                   GraphBaseObj::DiaTitel+
                   (DD->Mode & GraphBaseObj::DiaRahmen),
                   str.c_str(),DD->NumDigits);
          DD->XAxis = AkXAxis;
          DD->YAxis = AkYAxis;
          DDiagData.PostProcTerm = wxT("4\\pi*y");
        }
        else
        {
          wxString str(_("Fourier summing-Values 0 to 1"));
          DD->Init(DarstDataObj::realY,0,
                   GraphBaseObj::DiaTitel+
                   (DD->Mode & GraphBaseObj::DiaRahmen),
                   str.c_str(),DD->NumDigits);
          DD->XAxis = AkXAxis;
          DD->YAxis = AkYAxis;
          DDiagData.PostProcTerm = wxT("2\\pi*y+1\\2");
        }
        PostProcFunktion->Term.SetStr(DDiagData.PostProcTerm.c_str());
        DDiagData.SetPostProc(1);
        DDiagData.SetAuto(DarstDataObj::AutoY |
                          (DDiagData.AutoMode & ~DarstDataObj::AutoX));
        UpdateWerBer();
      }
      DDiagData.Ma = FD.Ma;
      if (MyMacroRecorder::Get()->IsRecording())
      {
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_MODIFYFOURIERSETTINGS"),GetId());
        DiagDataBaseObj *tmp = dynamic_cast<DiagDataBaseObj *>(&DDiagData);
        tmp->XmlSpeichern(MyMacroRecorder::Get()->GetAkNode(),&orig);
      }
    }
    SendModified();
  }
}

void TEFGraphControl::OnGraphLaden(wxCommandEvent &)
{
  GraphladenData GD;
  TEFGraphladenDlg *GL;
  int RetStatus;

  GD = wxGetApp().m_graphdata;

  GL = new TEFGraphladenDlg(GD);
  if (GL)
  {
    GL->Create(wxGetApp().GetTopWindow(),dlgGraphladen,_("Load graph"));
    RetStatus = GL->ShowModal();
    GL->Destroy();
    wxBeginBusyCursor();
    if ((RetStatus == wxID_OK) && (GD.FileName[0] != 0))
    {
      AddUndo(UndoDDiagData);
      NumListFormat tmpLDV = DDiagData.YDaten.GetLDV();
      tmpLDV.DecSeparator.SetStr(GD.DecSeparator);
      tmpLDV.ListSeparator.SetStr(GD.ListSeparator);
      DDiagData.YDaten.LoadGraph(GD.FileName.c_str(),0,
                                 (GD.Mode & GraphladenData::GraphSortieren) > 0,
                                 true,
                                 (GD.Mode & GraphladenData::GraphInDatei) > 0,
                                 true,
                                 GD.ImaginaryDataIncluded,GD.HeaderIncluded,&tmpLDV,
                                 GD.XValuesIncluded,GD.XValuesColumn,
                                 GD.TabSeparator,GD.SpaceSeparator);
    }
    wxEndBusyCursor();
  }
  UpdateWerBer();
  SendModified();
}

void TEFGraphControl::OnGraphSpeichern(wxCommandEvent &)
{
  GraphspeichernData GD;
  TEFGraphspeichernDlg *GL;
  int RetStatus;

  if (DDiagData.YDaten.YDatenVorhanden())
  {
    GD.Mode = (unsigned short)wxGetApp().GraphWriteForm;
    GL = new TEFGraphspeichernDlg(GD);
    if (GL)
    {
      GL->Create(wxGetApp().GetTopWindow(),dlgGraphspeichern,_("Save graph"));
      RetStatus = GL->ShowModal();
      GL->Destroy();
      if ((RetStatus == wxID_OK) && (GD.FileName[0] != 0))
      {
        wxGetApp().GraphWriteForm = GD.Mode;
        if (!_tcsstr(GD.FileName.c_str(),wxT(".")))
          GD.FileName.AddStr(wxT("grd"));
        wxBeginBusyCursor();
        bool ret = DDiagData.YDaten.WriteGraph
                     (GD.FileName.c_str(),0,
                     (GD.Mode & GraphspeichernData::GraphGRD) > 0,
                     (GD.Mode & GraphspeichernData::GraphTab) > 0,
                     (GD.Mode & GraphspeichernData::GraphKomma) > 0,
                     (GD.Mode & GraphspeichernData::GraphIngForm) > 0);
        wxEndBusyCursor();
        if (!ret)
        {
          wxString AZeile;
          AZeile.Printf(_("Error during graph %s save !"),GD.FileName.c_str());
          wxMessageBox(AZeile,wxT("Easy Funktion"),wxOK+wxICON_EXCLAMATION,
                       wxGetApp().GetTopWindow());
        }
      }
    }
  }
}

void TEFGraphControl::OnLegendeOben(wxCommandEvent &event)
{
  AddUndo(UndoDiagDataBase);
  if (event.IsChecked())
  {
    DDiagData.AutoMode |= DarstDataObj::dLegendeOben;
    UpdateWerBer();
  }
  else
    DDiagData.AutoMode &= ~DarstDataObj::dLegendeOben;
  DDiagData.LegX = 0;
  DDiagData.LegY = 0;
  Invalidate();
  SendModified();
}

void TEFGraphControl::SetActiveGraph(int graph, wxDC &DC)
{
  GetAkDiag().NaechsteFunktion(graph,
                               DDiagData.Legende.Color,DC);
  if (MyMacroRecorder::Get()->IsRecording() &&
      (graph != EFDiagObj::NotMoveCursor) &&
      (DDiagData.AkGraph != graph))

  {
    MyMacroRecorder::Get()->AddEvent(wxT("CMD_SELECTGRAPH"),GetId());
    MGPStr tmp;
    tmp.AddInt(graph);
    MyMacroRecorder::Get()->GetAkNode()->AddProperty(wxT("index"),tmp.c_str());
  }
}

void TEFGraphControl::OnMacroDarstellung(wxCommandEvent &event)
{
  AddUndo(UndoDiagDataBase);
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  DDiagData.XmlLaden(node);
  PostProcFunktion->Term.SetStr(DDiagData.PostProcTerm.c_str());
  DDiagData.SetPostProc(DDiagData.GetPostProc());
  if (event.GetId() == XRCID("CMD_ADDLABEL"))
  {
    DDiagData.Labels.Begin();
    while (DDiagData.Labels.NextEint());
    wxClientDC DC(this);
    SetScreenParams(DC);
    LabelSizeNDraw(DC,DDiagData.Labels.GetAkEint(),0,0,false);
    LabelIntoSight(DDiagData.Labels.GetAkEint());
  }
  UpdateWerBer();
}

void TEFGraphControl::OnMacroGraphOptions(wxCommandEvent &event)
{
  AddUndo(UndoDDiagData);
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  GraphDataObj tmp(DDiagData.YDaten.GetAkEint());
  tmp.XmlLoadGraph(node);
  EFXmlProperty *prop = node->GetProperties();
  int ret;
  MyMacroRecorder::Get()->GetPropertyInt(prop,wxT("allgraphs"),ret);
  bool allgraphs = (ret != 0);
  if (event.GetId() == XRCID("CMD_ADDGRAPH"))
    DDiagData.YDaten.AppendYFeld(&tmp);
  else
    DDiagData.YDaten.ModifyGraph(&tmp,allgraphs,true);
  DDiagData.XmlLoadLegend(node); // Load legend
  UpdateWerBer();
}

void TEFGraphControl::OnMacroSelectLabel(wxCommandEvent &event)
{
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  EFXmlProperty *prop = node->GetProperties();
  int index;
  MyMacroRecorder::Get()->GetPropertyInt(prop,wxT("index"),index);
  SetActiveLabel(index);
}

void TEFGraphControl::OnMacroSelectDiagram(wxCommandEvent &event)
{
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  EFXmlProperty *prop = node->GetProperties();
  int index;
  MyMacroRecorder::Get()->GetPropertyInt(prop,wxT("index"),index);
  unsigned char newdiag = (unsigned char)index;
  wxClientDC DC(this);
  SetScreenParams(DC);
  MarkDiag(wxPoint(0,0),DC,true,&newdiag);
}

void TEFGraphControl::OnMacroSelectGraph(wxCommandEvent &event)
{
  wxClientDC DC(this);
  SetScreenParams(DC);
  DC.SetFont(*LegendeFont);
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  EFXmlProperty *prop = node->GetProperties();
  int graph;
  MyMacroRecorder::Get()->GetPropertyInt(prop,wxT("index"),graph);
  if (m_hasfocus)
    DrawSizeRect(DC); // Size-Rechtecke verstecken
  SetMarkersVisible(false,DC);
  GetAkDiag().NaechsteFunktion(graph,
                               DDiagData.Legende.Color,DC);
  SetMarkersVisible(true,DC);
  DDiagData.AkGraph = DDiagData.YDaten.GetAkPos();
  DiagCurPos(DC);
  DC.SetUserScale(1.0,1.0);
  if (m_hasfocus)
    DrawSizeRect(DC); // Size-Rechtecke verstecken
}

BEGIN_MACRO_EVENT_TABLE(TEFGraphControl,TEFTabControl, MyMacroRecorder)
  // Macro events
  EVT_MENU(MACROID("CMD_SELECTDIAGRAM"), TEFGraphControl::OnMacroSelectDiagram)
  EVT_MENU(MACROID("CMD_ADDGRAPH"), TEFGraphControl::OnMacroGraphOptions)
  EVT_MENU(MACROID("CMD_MODIFYGRAPH"), TEFGraphControl::OnMacroGraphOptions)
  EVT_MENU(MACROID("CMD_SELECTGRAPH"), TEFGraphControl::OnMacroSelectGraph)
  EVT_MENU(MACROID("CMD_MODIFYDIAGRAM"), TEFGraphControl::OnMacroDarstellung)
  EVT_MENU(MACROID("CMD_MODIFYFOURIERSETTINGS"), TEFGraphControl::OnMacroDarstellung)
  EVT_MENU(MACROID("CMD_MODIFYLEGEND"), TEFGraphControl::OnMacroDarstellung)
  EVT_MENU(MACROID("CMD_ADDLABEL"), TEFGraphControl::OnMacroDarstellung)
  EVT_MENU(MACROID("CMD_MODIFYLABEL"), TEFGraphControl::OnMacroDarstellung)
  EVT_MENU(MACROID("CMD_SELECTLABEL"), TEFGraphControl::OnMacroSelectLabel)

  // Clipboard events
  EVT_MENU(wxID_PASTE,TEFGraphControl::OnEditPaste) // necessary for PopupMenu() and Macro (?!)
  EVT_MENU(wxID_COPY,TEFGraphControl::OnEditCopy)
  EVT_UPDATE_UI(wxID_COPY,TEFGraphControl::CeEditCopy)
  EVT_UPDATE_UI(wxID_CUT,TEFGraphControl::CeEditCut)

  // General window events
  EVT_SIZE(TEFGraphControl::OnSize)
  EVT_MOVE(TEFGraphControl::OnMove)
  EVT_PAINT(TEFGraphControl::OnPaint)
  EVT_KEY_DOWN(TEFGraphControl::OnKeyDown)
  EVT_KILL_FOCUS(TEFGraphControl::OnKillFocus)
  EVT_SET_CURSOR(TEFGraphControl::OnSetCursor)
  EVT_SET_FOCUS(TEFGraphControl::OnSetFocus)
  EVT_MOTION(TEFGraphControl::OnMouseMove)
  EVT_LEFT_UP(TEFGraphControl::OnLButtonUp)
  EVT_LEFT_DOWN(TEFGraphControl::OnLButtonDown)
  EVT_RIGHT_UP(TEFGraphControl::OnContextMenu)
  EVT_LEFT_DCLICK(TEFGraphControl::OnLButtonDblClk)

  // Menu activation events
  EVT_UPDATE_UI(XRCID("CMD_ZOOMIN"), TEFGraphControl::CeHasFocus)
  EVT_UPDATE_UI(XRCID("CMD_GRAPHMENU"), TEFGraphControl::CeHasFocus)
  EVT_UPDATE_UI(XRCID("CMD_LABELMENU"), TEFGraphControl::CeHasFocus)
  EVT_UPDATE_UI(XRCID("CMD_DIAGPROPMENU"), TEFGraphControl::CeHasFocus)
  EVT_UPDATE_UI(XRCID("CMD_MARKMENU"), TEFGraphControl::CeGraph)

  EVT_MENU(MACROID("CMD_SECONDDIAGRAM"), TEFGraphControl::OnDiag2)
  EVT_UPDATE_UI(XRCID("CMD_SECONDDIAGRAM"),TEFGraphControl::CeDiag2)

  EVT_MENU(MACROID("CMD_POSTPROC"), TEFGraphControl::OnPostProc)
  EVT_UPDATE_UI(XRCID("CMD_POSTPROC"),TEFGraphControl::CePostProc)

  EVT_MENU(MACROID("CMD_AUTOX"), TEFGraphControl::OnAutoX)
  EVT_UPDATE_UI(XRCID("CMD_AUTOX"),TEFGraphControl::CeAutoX)

  EVT_MENU(MACROID("CMD_AUTOY"), TEFGraphControl::OnAutoY)
  EVT_UPDATE_UI(XRCID("CMD_AUTOY"),TEFGraphControl::CeAutoY)

  EVT_MENU(XRCID("CMD_INCREASE"), TEFGraphControl::OnIncrease)
  EVT_UPDATE_UI(XRCID("CMD_INCREASE"), TEFGraphControl::CeHasFocus)

  EVT_MENU(XRCID("CMD_DECREASE"), TEFGraphControl::OnDecrease)
  EVT_UPDATE_UI(XRCID("CMD_DECREASE"), TEFGraphControl::CeHasFocus)

  // Not macro recorded general diagram events
  EVT_MENU(XRCID("ICM_DELDIAG"), TEFGraphControl::OnDelDiag)

  EVT_MENU(XRCID("ICM_FOURDIALOG"), TEFGraphControl::OnFourierDialog)
  EVT_UPDATE_UI(XRCID("ICM_FOURDIALOG"),TEFGraphControl::CeFourierDialog)

  EVT_MENU(XRCID("ICM_DIAGRAMOPTIONS"), TEFGraphControl::OnDarstellung)
  EVT_UPDATE_UI(XRCID("ICM_DIAGRAMOPTIONS"), TEFGraphControl::CeHasFocus)

  // Label events
  EVT_MENU(MACROID("CMD_DELLABEL"), TEFGraphControl::OnDelLabel)
  EVT_UPDATE_UI(XRCID("CMD_DELLABEL"), TEFGraphControl::CeHasFocus)

  EVT_MENU(MACROID("CMD_DELALLLABELS"),TEFGraphControl::OnDelAllLabels)
  EVT_UPDATE_UI(XRCID("CMD_DELALLLABELS"),TEFGraphControl::CeLabels)

  EVT_MENU(MACROID("CMD_SHOWLABELS"),TEFGraphControl::OnLabelsIntoSight)
  EVT_UPDATE_UI(XRCID("CMD_SHOWLABELS"),TEFGraphControl::CeLabels)

  EVT_MENU(MACROID("CMD_MOVELABEL"), TEFGraphControl::OnMoveLabel)
  EVT_UPDATE_UI(XRCID("CMD_MOVELABEL"),TEFGraphControl::CeMoveLabel)

  // Not macro recorded label events
  EVT_MENU(XRCID("ICM_LABELOPTIONS"), TEFGraphControl::OnLabelOptions)
  EVT_UPDATE_UI(XRCID("ICM_LABELOPTIONS"), TEFGraphControl::CeHasFocus)

  EVT_MENU(XRCID("ICM_ADDLABEL"), TEFGraphControl::OnAddLabel)
  EVT_UPDATE_UI(XRCID("ICM_ADDLABEL"), TEFGraphControl::CeHasFocus)

  EVT_MENU(XRCID("ICM_SELECTLABEL"), TEFGraphControl::OnLabelSelect)
  EVT_UPDATE_UI(XRCID("ICM_SELECTLABEL"),TEFGraphControl::CeLabels)

  // Marker events
  EVT_MENU(MACROID("CMD_MARK"), TEFGraphControl::OnMarke)
  EVT_UPDATE_UI(XRCID("CMD_MARK"), TEFGraphControl::CeMarke)

  EVT_MENU(MACROID("CMD_PEAK"), TEFGraphControl::OnSpitze)
  EVT_UPDATE_UI(XRCID("CMD_PEAK"), TEFGraphControl::CeGraph)

  EVT_MENU(MACROID("CMD_MIN"), TEFGraphControl::OnMin)
  EVT_UPDATE_UI(XRCID("CMD_MIN"), TEFGraphControl::CeGraph)

  EVT_MENU(MACROID("CMD_MAX"), TEFGraphControl::OnMax)
  EVT_UPDATE_UI(XRCID("CMD_MAX"), TEFGraphControl::CeGraph)

  EVT_MENU(MACROID("CMD_LEFT"), TEFGraphControl::OnLinks)
  EVT_UPDATE_UI(XRCID("CMD_LEFT"), TEFGraphControl::CeGraph)

  EVT_MENU(MACROID("CMD_RIGHT"), TEFGraphControl::OnRechts)
  EVT_UPDATE_UI(XRCID("CMD_RIGHT"), TEFGraphControl::CeGraph)

  // Legend events
  EVT_MENU(MACROID("CMD_LEGENDONTOP"), TEFGraphControl::OnLegendeOben)
  EVT_UPDATE_UI(XRCID("CMD_LEGENDONTOP"),TEFGraphControl::CeLegendeOben)

  // Not macro recorded legend events
  EVT_MENU(XRCID("ICM_MOVELEGEND"), TEFGraphControl::OnMoveLegend)
  EVT_UPDATE_UI(XRCID("ICM_MOVELEGEND"),TEFGraphControl::CeMoveLegend)

  // Graph events
  EVT_MENU(MACROID("CMD_DELETEALLGRAPHS"), TEFGraphControl::OnAlleLoeschen)
  EVT_UPDATE_UI(XRCID("CMD_DELETEALLGRAPHS"), TEFGraphControl::CeGraph)

  EVT_MENU(MACROID("CMD_DELETEGRAPH"), TEFGraphControl::OnLoeschen)
  EVT_UPDATE_UI(XRCID("CMD_DELETEGRAPH"), TEFGraphControl::CeGraph)

  EVT_MENU(MACROID("CMD_GRAPHCOPY"), TEFGraphControl::OnCopyGraph)
  EVT_UPDATE_UI(XRCID("CMD_GRAPHCOPY"), TEFGraphControl::CeGraph)

  // Not macro recorded graph events
  EVT_MENU(XRCID("ICM_GRAPHVALUETABLE"), TEFGraphControl::OnGraphTable)
  EVT_UPDATE_UI(XRCID("ICM_GRAPHVALUETABLE"), TEFGraphControl::CeGraph)

  EVT_MENU(XRCID("ICM_GRAPHFROMVALUES"), TEFGraphControl::OnGraphWerte)
  EVT_UPDATE_UI(XRCID("ICM_GRAPHFROMVALUES"), TEFGraphControl::CeHasFocus)

  EVT_MENU(XRCID("ICM_LOADGRAPH"),TEFGraphControl::OnGraphLaden)
  EVT_UPDATE_UI(XRCID("ICM_LOADGRAPH"),TEFGraphControl::CeHasFocus)

  EVT_MENU(XRCID("ICM_SAVEGRAPH"),TEFGraphControl::OnGraphSpeichern)
  EVT_UPDATE_UI(XRCID("ICM_SAVEGRAPH"),TEFGraphControl::CeGraph)

  EVT_MENU(XRCID("ICM_GRAPHOPTIONS"), TEFGraphControl::OnGraphOptionen)
  EVT_UPDATE_UI(XRCID("ICM_GRAPHOPTIONS"), TEFGraphControl::CeGraph)

  // Undo/Redo events
  EVT_MENU(wxID_REDO, TEFGraphControl::OnEditRedo)
  EVT_UPDATE_UI(wxID_REDO, TEFGraphControl::CeEditRedo)

  EVT_MENU(wxID_UNDO, TEFGraphControl::OnEditUndo)
  EVT_UPDATE_UI(wxID_UNDO, TEFGraphControl::CeEditUndo)
END_EVENT_TABLE()


