// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efmacro.cpp
    \brief Source code for the macro recorder class
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h>

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>

#include <cstring>
#include <complex>
#include <limits>
#include <set>
#include <fstream>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagtype.h"
#include "efgemein.h"
#include "efmcxio.h"
#include "efresid.h"

#endif

#include <wx/xrc/xh_toolb.h>
#include <wx/xrc/xh_combo.h>
#include "efxmlio.h"
#include "efmacro.h"

// ----------------------------------------------------------------------------
// MyMacroRecorderCreator
// ----------------------------------------------------------------------------

MyMacroRecorderCreator MacroCreator;

// ----------------------------------------------------------------------------
// MyMacroRecorder
// ----------------------------------------------------------------------------

int MyMacroRecorder::GetXrcId(const char *name,int id)
{
  ms_CurrentList->NeuAkEint();
  ms_CurrentList->GetAkEint().m_Name = ConvertStr(name);
  if (!id)
    id = EFXmlResource::Get()->GetXRCID(ConvertStr(name).c_str());
  ms_CurrentList->GetAkEint().m_XRCId = id;
  return id;
}

MacroCommandList *MyMacroRecorder::ms_CurrentList;

MacroCommandList *MyMacroRecorder::ms_ViewCommandList;

MacroCommandList *MyMacroRecorder::ms_GraphCCommandList;

MacroCommandList *MyMacroRecorder::ms_EditCommandList;

MacroCommandList *MyMacroRecorder::ms_ManualCommandList;

MyMacroRecorder *MyMacroRecorder::ms_OrgInstance = 0;

MyMacroRecorder *MyMacroRecorder::ms_Instance = 0;

MyMacroRecorder::MyMacroRecorder()
{
  m_Stepping = false;
  m_Locked = false;
  m_AkNode = 0;
  m_Playback = false;
  m_Recording = false;
}

void MyMacroRecorder::SaveMacroList(const char *fname)
{
  fstream file(fname,ios::out);
  MacroCommandList *cur = 0;
  while ((cur = GetNextList(cur)) != 0)
  {
    cur->Begin();
    if (cur->GetMaxEintrag() >= 0)
      do
      {
        wxString tmp;
        tmp += wxT("'");
        tmp += cur->GetAkEint().m_Name.c_str();
        tmp += wxT("' ");
        wxString tmp1;
        tmp1.Printf(wxT("%i"),cur->GetAkEint().m_XRCId);
        tmp += tmp1;
        tmp += wxT("\n");
        const char *tmpcs = tmp.mb_str();
        file << tmpcs;
      }
      while (cur->NextEint());
  }
}

bool MyMacroRecorder::GetPropertyInt(EFXmlProperty *prop,const wxChar *name,int &ret)
{
  while (prop)
  {
    if (!wxStrcmp(name,prop->GetName().c_str()))
    {
      ret = _ttoi(prop->GetValue().c_str());
      return true;
    }
    prop = prop->GetNext();
  }
  return false;
}

EFXmlProperty *MyMacroRecorder::GetProperty(EFXmlProperty *prop,const wxChar *name)
{
  while (prop)
  {
    if (!wxStrcmp(name,prop->GetName().c_str()))
      return prop;
    prop = prop->GetNext();
  }
  return 0;
}

void MyMacroRecorder::AddPos(int x1,int y1,int x2,int y2)
{
  wxString tmp1;
  tmp1.Printf(wxT("%i"),x1);
  m_AkNode->AddProperty(wxT("startx"),tmp1);
  tmp1.Printf(wxT("%i"),y1);
  m_AkNode->AddProperty(wxT("starty"),tmp1);
  tmp1.Printf(wxT("%i"),x2);
  m_AkNode->AddProperty(wxT("endx"),tmp1);
  tmp1.Printf(wxT("%i"),y2);
  m_AkNode->AddProperty(wxT("endy"),tmp1);
}

bool MyMacroRecorder::GetPos(EFXmlProperty *prop,int &x1,int &y1,int &x2,int &y2)
{
  x1 = y1 = x2 = y2 = 0;
  while (prop)
  {
    const MGPStr &tmp = prop->GetValue();
    if (!wxStrcmp(prop->GetName().c_str(),wxT("startx")))
      x1 = _ttoi(tmp.c_str());
    else
    if (!wxStrcmp(prop->GetName().c_str(),wxT("starty")))
      y1 = _ttoi(tmp.c_str());
    else
    if (!wxStrcmp(prop->GetName().c_str(),wxT("endx")))
      x2 = _ttoi(tmp.c_str());
    else
    if (!wxStrcmp(prop->GetName().c_str(),wxT("endy")))
      y2 = _ttoi(tmp.c_str());
    prop = prop->GetNext();
  }
  return x1 || x2 || y1 || y2;
}

void MyMacroRecorder::InsertString(const wxChar *str,int Id,int Pos,int Zeile)
{
  int LastPos = Pos,LastZeile = Zeile;
  int i = 0;
  do
  {
    int anfpos = i;
    while ((str[i] != '\n') && (str[i] != '\r') && str[i])
      i++;
    Pos += (i-anfpos);
    AddEvent(wxT("CMD_EDITINSERT"),Id);
    AddPos(LastPos,LastZeile,Pos,Zeile);
    MGPStr tmp;
    tmp = &str[anfpos];
    tmp.ShortStr(i-anfpos);
    new EFXmlNode(m_AkNode,EFXmlNode::TEXT_NODE,_TEXT(""),tmp.c_str(),0,0);
    if (str[i])
    {
      AddEvent(wxT("CMD_EDITRETURN"),Id);
      Zeile++;
      Pos = 0;
    }
    // Support for DOS linebreaks
    if ((str[i] == '\r') && (str[i+1] == '\n'))
      i++;
    if (str[i])
      i++;
    LastPos = Pos;
    LastZeile = Zeile;
  }
  while (str[i]);
}

void MyMacroRecorder::InsertChar(const wxChar chr,int Id,int Pos,int Zeile)
{
  int LastPos = Pos,LastZeile = Zeile;
  if (chr != '\n')
    LastPos--;
  else
    LastZeile--;

  EFXmlProperty *prop = 0;
  if (m_AkNode)
    prop = m_AkNode->GetProperties();
  if (!prop || wxStrcmp(m_AkNode->GetName().c_str(),wxT("CMD_EDITINSERT")) ||
      (_ttoi(GetProperty(prop,wxT("window"))->GetValue().c_str()) != Id) ||
      (_ttoi(GetProperty(prop,wxT("endy"))->GetValue().c_str()) != LastZeile) ||
      (_ttoi(GetProperty(prop,wxT("endx"))->GetValue().c_str()) != LastPos) &&
      (chr != 13))
  {
    AddEvent(wxT("CMD_EDITINSERT"),Id);
    AddPos(LastPos,LastZeile,Pos,Zeile);
    new EFXmlNode(m_AkNode,EFXmlNode::TEXT_NODE,_TEXT(""),_TEXT(""),0,0);
  }
  else
  {
    MGPStr tmp;
    tmp.AddInt(Zeile);
    GetProperty(prop,wxT("endy"))->SetValue(tmp);
    tmp = wxT("");
    tmp.AddInt(Pos);
    GetProperty(prop,wxT("endx"))->SetValue(tmp);
  }
  EFXmlNode *snode = m_AkNode->GetChildren();
  MGPStr tmp = snode->GetContent();
  tmp.AddChar(chr);
  snode->SetContent(tmp);
}

void MyMacroRecorder::RecordMacro()
{
  m_Doc.SetRoot(new EFXmlNode(0,EFXmlNode::ELEMENT_NODE,
                              _TEXT("Easy_Funktion_Macro"),
                              _TEXT(""),0,0));
  m_Doc.GetRoot()->AddProperty(wxT("version"),wxT("1.0"));
  m_Recording = true;
  m_Playback = false;
  m_RepeatCount = 0;
}

bool MyMacroRecorder::PlaySubMacro(const char *name)
{
  if (!ms_OrgInstance)
  {
    ms_OrgInstance = ms_Instance;
    ms_Instance = new MyMacroRecorder;
    return ms_Instance->PlayMacro(name);
  }
  else
    return false;  
}

EFXmlNode *MyMacroRecorder::GetAkNode()
{
  if (!m_AkNode && ms_OrgInstance)
  {
    delete ms_Instance;
    ms_Instance = ms_OrgInstance;
    ms_OrgInstance = NULL;
    ms_Instance->m_AkNode = ms_Instance->m_AkNode->GetNext();
    ms_CurrentList = ms_ManualCommandList;
    return ms_Instance->m_AkNode;
  }
  return m_AkNode;
}

EFXmlNode *MyMacroRecorder::GetNextNode()
{
  if (m_AkNode)
    m_AkNode = m_AkNode->GetNext();
  return m_AkNode;
}

bool MyMacroRecorder::PlayMacro(const char *name)
{
  m_Doc.SetRoot(0);
  ifstream file(name,ios::in);
#if !wxUSE_UNICODE
  m_Doc.Load(file,wxLocale::GetSystemEncodingName().c_str());
#else
  m_Doc.Load(file);
#endif
  m_AkNode = m_Doc.GetRoot();
  m_RepeatCount = 0;
  m_Stepping = false;
  if (m_AkNode && m_AkNode->GetProperties())
  {
    int ret;
    if (GetPropertyInt(m_AkNode->GetProperties(),wxT("repeatcount"),ret))
      m_RepeatCount = ret;
    if (GetPropertyInt(m_AkNode->GetProperties(),wxT("stepping"),ret))
      m_Stepping = (ret > 0);
  }
  if (m_AkNode)
    m_AkNode = m_AkNode->GetChildren();
  m_Playback = m_AkNode != 0;
  m_Recording = false;
  return m_Playback;
}

bool MyMacroRecorder::EndMacro(const char *name)
{
  if (m_RepeatCount)
  {
    m_RepeatCount--;
    m_AkNode = m_Doc.GetRoot();
    if (m_AkNode)
      m_AkNode = m_AkNode->GetChildren();
    return false;
  }
  if (name && name[0])
  {
    ofstream file(name,ios::out);
    m_Doc.Save(file);
  }
  m_Doc.SetRoot(0);
  m_AkNode = 0;
  m_Playback = false;
  m_Recording = false;
  return true;
}

void MyMacroRecorder::InitList(const char *name)
{
  if (!strcmp(name,"TEFView"))
  {
    ms_ViewCommandList = new MacroCommandList;
    ms_CurrentList = ms_ViewCommandList;
  }
  else
  if (!strcmp(name,"TEFGraphControl"))
  {
    ms_GraphCCommandList = new MacroCommandList;
    ms_CurrentList = ms_GraphCCommandList;
  }
  else
  if (!strcmp(name,"TEFEdit"))
  {
    ms_EditCommandList = new MacroCommandList;
    ms_CurrentList = ms_EditCommandList;
  }
}

void MyMacroRecorder::AddEvent(const wxChar *name,int Id)
{
  if (!m_Recording || m_Playback)
    return;
  m_Doc.GetRoot();
  m_AkNode = new EFXmlNode(m_Doc.GetRoot(),EFXmlNode::ELEMENT_NODE,
                           name,_TEXT(""),0,0);
  wxString tmp1;
  tmp1.Printf(wxT("%i"),Id);
  m_AkNode->AddProperty(wxT("window"),tmp1);
}

void MyMacroRecorder::AddEvent(wxEvent &evt,int Id)
{
  if (!m_Recording || m_Playback || m_Locked)
    return;
  wxCommandEvent *event = wxDynamicCast(&evt,wxCommandEvent);
  if (!event || (event->GetEventType() != wxEVT_COMMAND_MENU_SELECTED))
    return;
  MacroCommandList *tmp = 0;
  while ((tmp = GetNextList(tmp)) != 0)
  {
    tmp->Begin();
    if (tmp->GetMaxEintrag() >= 0)
      do
      {
        if ((tmp->GetAkEint().m_XRCId == event->GetId()) &&
             (!wxStrncmp(tmp->GetAkEint().m_Name.c_str(),wxT("CMD_"),4) ||
              !wxStrncmp(tmp->GetAkEint().m_Name.c_str(),wxT("wxID_"),5)))
        {
          m_Doc.GetRoot();
          m_AkNode = new EFXmlNode(m_Doc.GetRoot(),EFXmlNode::ELEMENT_NODE,
                                   tmp->GetAkEint().m_Name.c_str(),_TEXT(""),0,0);
          if (Id)
          {
            wxString tmp1;
            tmp1.Printf(wxT("%i"),Id);
            m_AkNode->AddProperty(wxT("window"),tmp1);
          }
          if (event->IsChecked())
            m_AkNode->AddProperty(wxT("checked"),wxT("1"));
          return;
        }
      }
      while (tmp->NextEint());
  }
}

