// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efprint.cpp
    \brief Source code for the printout class
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>
#include <wx/dnd.h>

#include <wx/print.h>
#include <wx/docview.h>

#include <complex>
#include <limits>
#include <cstdio>
#include <ctime>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "efprint.h"
#include "ef.h"
#include "efresid.h"
#include "efedit.h"
#include "efeditsr.h"
#include "efdocum.h"
#include "eftabcon.h"
#include "efgraphc.h"
#include "efsolver.h"
#include "efview.h"

#endif

// ----------------------------------------------------------------------------
// EFPrintOut
// ----------------------------------------------------------------------------

EFPrintOut::EFPrintOut(wxWindow *window, const wxString& title)
             : wxPrintout(title)
{
  Window = window;
}

void EFPrintOut::GetPageInfo (int *minPage, int *maxPage,
                              int *selFromPage, int *selToPage)
{
  TEFEdit *Edit = wxDynamicCast(Window,TEFEdit);
  if (Edit)
  {
    BeginPrinting(0);
    OnEndPrinting();

    size_t count = 0;
    for (size_t i = 1;i < Pages.size();i++)
      if (Pages[i])
        count++;
    *minPage = 1;
    *maxPage = count;
    *selFromPage = 0;
    *selToPage = count;
  }
  else
  {
    *minPage = *maxPage = 1;
    *selFromPage = *selToPage = 0;
  }
}

void EFPrintOut::OnBeginPrinting()
{
  wxPrintout::OnBeginPrinting();
  wxDC *DC = GetDC();
  BeginPage(DC);
  BeginPrinting(DC);
}

void EFPrintOut::BeginPrinting(wxDC *DC)
{
  TEFEdit *Edit = wxDynamicCast(Window,TEFEdit);
  NumPages = 1;
  if (Edit)
  {
    wxDC *dc = 0;
    if (!DC) // fuer GetPageInfo, ClientDC verwenden !
    {
      dc = new wxClientDC(Window);
      DC = dc;
    }
    int fHeight = Edit->SetPrintDocSize(*DC);
    int i = 1;
    int AkLine = 1,OldAkLine;
    bool Visible;
    NumPages = 0;
    do
    {
      NumPages++;
      OldAkLine = AkLine;
      AkLine = Edit->NextPg(fHeight,i,&Visible);
      if (dc) // nur bei PageInfo Seiten ermitteln !
      {
        Pages.resize(NumPages+1);
        Pages[NumPages] = Visible ||
                          Edit->VisibleChildren(i,OldAkLine,AkLine,fHeight);
      }
    }
    while (AkLine > 0);
    delete dc;
  }
  else
  {
    Pages.resize(NumPages+1);
    Pages[NumPages] = true;
  }  
}


void EFPrintOut::OnEndPrinting()
{
  TEFEdit *Edit = wxDynamicCast(Window,TEFEdit);
  if (Edit) Edit->SetScreenDocSize();
  wxPrintout::OnEndPrinting();
}

void EFPrintOut::BeginPage (wxDC *dc)
{
  // Get the logical pixels per inch of screen and printer
  int ppiScreenX, ppiScreenY;
  GetPPIScreen(&ppiScreenX, &ppiScreenY);
  int ppiPrinterX, ppiPrinterY;
  GetPPIPrinter(&ppiPrinterX, &ppiPrinterY);

  // This scales the DC so that the printout roughly represents the
  // the screen scaling. The text point size _should_ be the right size
  // but in fact is too small for some reason. This is a detail that will
  // need to be addressed at some point but can be fudged for the
  // moment.
  double scalex = double(ppiPrinterX)/double(ppiScreenX);
  double scaley = double(ppiPrinterY)/double(ppiScreenY);

  // Now we have to check in case our real page size is reduced
  // (e.g. because we're drawing to a print preview memory DC)
  int pageWidth,pageHeight;
  int w, h;
  dc->GetSize(&w, &h);
  GetPageSizePixels(&pageWidth, &pageHeight);

  // If printer pageWidth == current DC width, then this doesn't
  // change. But w might be the preview bitmap width, so scale down.
  double overallScalex = scalex * double(w)/double(pageWidth);
  double overallScaley = scaley * double(h)/double(pageHeight);

  dc->SetUserScale(overallScalex, overallScaley);
}

void EFPrintOut::TitelZeile(wxDC *DC,int width,int /*height*/,int page)
{
  DC->SetTextForeground(*wxBLACK);
  // Kopfzeile ausgeben
  wxFont font(12,wxSWISS,wxNORMAL,wxNORMAL,false,wxT("Arial"));
  DC->SetFont(font);
  int rand = width/40;
  wxCoord w,h;
  DC->GetTextExtent(GetTitle(),&w,&h);
  DC->DrawText(GetTitle(),width/2-w/2,rand); // Dokument-Titel mittig ausgeben
  time_t timer;
  struct tm *TS;
  timer = time(NULL);
  TS = localtime(&timer);
  _TCHAR Zeile[200];
  _stprintf(Zeile,wxT("%02hi.%02hi.%4i"),
            TS->tm_mday,TS->tm_mon+1,TS->tm_year+1900);
  DC->GetTextExtent(Zeile,&w,&h);
  DC->DrawText(Zeile,width-rand-w,rand); // Datum rechts ausgeben
  wxString str;
  str.Printf(_("Page %i"),page);
  DC->DrawText(str,rand,rand); // Seitennummer links ausgeben
  DC->SetFont(wxNullFont);
}

bool EFPrintOut::OnPrintPage(int page)
{
  wxDC *DC = GetDC();

  double x,y;
  DC->GetUserScale(&x,&y);
  BeginPage(DC);

  TEFGraphControl *EFGraph = wxDynamicCast(Window,TEFGraphControl);
  TEFEdit *EFEdit = wxDynamicCast(Window,TEFEdit);
  wxResizeableControl *EFSzCn = wxDynamicCast(Window,wxResizeableControl);
  int w,h,xs,ys;
  GetPageSizeMM(&w,&h);
  GetPPIScreen(&xs,&ys);
  w = int(float(w*xs)/25.4);
  h = int(float(h*ys)/25.4);
  if (EFEdit)
  {
    int AkPg = 0,i = 0;
    while ((i < page) && (AkPg < (int)Pages.size()-1))
    {
      AkPg++;
      if (Pages[AkPg])
        i++;
    }
    EFEdit->PaintPage(*DC,AkPg);
  }
  else
  if (EFGraph)
    EFGraph->PrintDiag(*DC,w,h);
  else
  if (EFSzCn)
  {
    DC->SetDeviceOrigin(int(x*w/10),int(y*h/10));
    double rat = EFSzCn->GetRatio();
    wxSize PSize;
    if (w/double(h) > rat)
    {
      PSize.y = int(0.8*h);
      PSize.x = int(PSize.y*rat);
    }
    else
    {
      PSize.x = int(0.8*w);
      PSize.y = int(PSize.x/rat);
    }
    EFSzCn->Paint(*DC,true,PSize);
    DC->SetDeviceOrigin(0,0);
  }
  TitelZeile(DC,w,h,page);
  DC->SetUserScale(x,y);
  return true;
}

bool EFPrintOut::HasPage (int pageNumber)
{
  size_t count = 0;
  for (size_t i = 1;i < Pages.size();i++)
    if (Pages[i])
      count++;
  return (pageNumber >= 1) && (pageNumber <= (int)count);
}


