// Version 6.05 21.06.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efxrcres.cpp
    \brief Source code for the XRC based resources
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xh_toolb.h>
#include <wx/xrc/xh_combo.h>

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>

#include <complex>
#include <limits>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif
#include <wx/dnd.h>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "diagtype.h"
#include "cpxpsr.h"
#include "efgemein.h"
#include "efresid.h"
#include "ef.h"

#endif

#include <wx/toolbar.h>
#include <wx/combobox.h>
#include "efxrcres.h"

// ----------------------------------------------------------------------------
// EFZoomBoxXmlHandler
// ----------------------------------------------------------------------------

EFZoomBoxXmlHandler::EFZoomBoxXmlHandler()
 : wxComboBoxXmlHandler()
{
}

wxObject *EFZoomBoxXmlHandler::DoCreateResource()
{
    EFZoomBox *cmb = new EFZoomBox(m_parentAsWindow,GetID(),wxT(""),
                                   GetPosition(),GetSize());
    SetupWindow(cmb);

    return cmb;
}

bool EFZoomBoxXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("EFZoomBox"));
}

// ----------------------------------------------------------------------------
// EFToolBarXmlHandler
// ----------------------------------------------------------------------------

wxObject *EFToolBarXmlHandler::DoCreateResource()
{
    if (m_class == wxT("tool"))
    {
        m_toolbar->AddTool(GetID(),
                           GetBitmap(wxT("bitmap")),
                           GetBitmap(wxT("bitmap2")),
                           GetBool(wxT("toggle")),
                           GetPosition().x,
                           GetPosition().y,
                           NULL,
                           GetText(wxT("tooltip")),
                           GetText(wxT("longhelp")));
        return m_toolbar; // must return non-NULL
    }

    else
    if (m_class == wxT("separator"))
    {
        m_toolbar->AddSeparator();
        return m_toolbar; // must return non-NULL
    }

    else /*<object class="wxToolBar">*/
    {

        int style = GetStyle(wxT("style"), wxNO_BORDER | wxTB_HORIZONTAL);

#ifdef __WXMSW__
        if (!(style & wxNO_BORDER))
          style |= wxNO_BORDER;
#endif

        wxToolBar *toolbar = new wxToolBar(m_parentAsWindow,
                                    GetID(),
                                    GetPosition(),
                                    GetSize(),
                                    style,
                                    GetName());

        wxSize bmpsize = GetSize(wxT("bitmapsize"));
        if (!(bmpsize == wxDefaultSize))
            toolbar->SetToolBitmapSize(bmpsize);
        wxSize margins = GetSize(wxT("margins"));
        if (!(margins == wxDefaultSize))
            toolbar->SetMargins(margins.x, margins.y);
        long packing = GetLong(wxT("packing"), -1);
        if (packing != -1)
            toolbar->SetToolPacking(packing);
        long separation = GetLong(wxT("separation"), -1);
        if (separation != -1)
            toolbar->SetToolSeparation(separation);

        wxXmlNode *children_node = GetParamNode(wxT("object"));
        if (!children_node)
           children_node = GetParamNode(wxT("object_ref"));

        if (children_node == NULL) return toolbar;

        m_isInside = TRUE;
        m_toolbar = toolbar;

        wxXmlNode *n = children_node;

        while (n)
        {
            if ((n->GetType() == wxXML_ELEMENT_NODE) &&
                (n->GetName() == wxT("object") || n->GetName() == wxT("object_ref")))
            {
                wxObject *created = CreateResFromNode(n, toolbar, NULL);
                wxControl *control = wxDynamicCast(created, wxControl);
                if (!IsOfClass(n, wxT("tool")) &&
                    !IsOfClass(n, wxT("separator")) &&
                    control != NULL)
                    toolbar->AddControl(control);
            }
            n = n->GetNext();
        }

        m_isInside = FALSE;
        m_toolbar = NULL;

// Cannot realize toolbars here, as we may add controls later
        toolbar->Realize();

        return toolbar;
    }
}

bool EFToolBarXmlHandler::CanHandle(wxXmlNode *node)
{
    return ((!m_isInside && IsOfClass(node, wxT("EFToolBar"))) ||
            (m_isInside && IsOfClass(node, wxT("tool"))) ||
            (m_isInside && IsOfClass(node, wxT("separator"))));
}

// ----------------------------------------------------------------------------
// EFXmlResource
// ----------------------------------------------------------------------------

EFXmlResource *EFXmlResource::ef_instance = NULL;

